@@ -36,15 +36,17 @@ from sabnzbd.misc import match_str
 
 # Check for correct SABYenc version
 SABYENC_VERSION = None
+SABYENC_SIMD = None
 try:
     import sabyenc3
 
     SABYENC_ENABLED = True
     SABYENC_VERSION = sabyenc3.__version__
+    SABYENC_SIMD = sabyenc3.simd
     # Verify version to at least match minor version
     if SABYENC_VERSION[:3] != SABYENC_VERSION_REQUIRED[:3]:
         raise ImportError
-except ImportError:
+except:
     SABYENC_ENABLED = False
 
 
@@ -66,13 +68,15 @@ class Decoder:
     """Implement thread-like coordinator for the decoders"""
 
     def __init__(self):
-        logging.debug("Initializing decoders")
+
         # Initialize queue and servers
         self.decoder_queue = queue.Queue()
 
         # Initialize decoders
+        decoders = cfg.num_simd_decoders()
+        logging.debug("Initializing %d decoder(s)", decoders)
         self.decoder_workers = []
-        for i in range(cfg.num_decoders()):
+        for _ in range(decoders):
             self.decoder_workers.append(DecoderWorker(self.decoder_queue))
 
     def start(self):
@@ -89,7 +93,7 @@ class Decoder:
     def stop(self):
         # Put multiple to stop all decoders
         for _ in self.decoder_workers:
-            self.decoder_queue.put((None, None))
+            self.decoder_queue.put((None, None, None))
 
     def join(self):
         # Wait for all decoders to finish
@@ -99,10 +103,9 @@ class Decoder:
             except:
                 pass
 
-    def process(self, article: Article, raw_data: List[bytes]):
-        # We use reported article-size, just like sabyenc does
-        sabnzbd.ArticleCache.reserve_space(article.bytes)
-        self.decoder_queue.put((article, raw_data))
+    def process(self, article: Article, raw_data: List[bytes], data_size: int):
+        sabnzbd.ArticleCache.reserve_space(data_size)
+        self.decoder_queue.put((article, raw_data, data_size))
 
     def queue_full(self) -> bool:
         # Check if the queue size exceeds the limits
@@ -110,29 +113,28 @@ class Decoder:
 
 
 class DecoderWorker(Thread):
-    """The actuall workhorse that handles decoding!"""
+    """The actual workhorse that handles decoding!"""
 
     def __init__(self, decoder_queue):
         super().__init__()
         logging.debug("Initializing decoder %s", self.name)
-
-        self.decoder_queue: queue.Queue[Tuple[Optional[Article], Optional[List[bytes]]]] = decoder_queue
+        self.decoder_queue: queue.Queue[Tuple[Optional[Article], Optional[List[bytes]], Optional[int]]] = decoder_queue
 
     def run(self):
         while 1:
             # Set Article and NzbObject objects to None so references from this
             # thread do not keep the parent objects alive (see #1628)
             decoded_data = raw_data = article = nzo = None
-            article, raw_data = self.decoder_queue.get()
+            article, raw_data, data_size = self.decoder_queue.get()
             if not article:
-                logging.info("Shutting down decoder %s", self.name)
+                logging.debug("Shutting down decoder %s", self.name)
                 break
 
             nzo = article.nzf.nzo
             art_id = article.article
 
             # Free space in the decoder-queue
-            sabnzbd.ArticleCache.free_reserved_space(article.bytes)
+            sabnzbd.ArticleCache.free_reserved_space(data_size)
 
             # Keeping track
             article_success = False
@@ -213,7 +215,7 @@ class DecoderWorker(Thread):
                 elif not article_success:
                     # If not pre-check, this must be a bad article
                     if not nzo.precheck:
-                        logging.info("Badly formed yEnc article in %s", art_id, exc_info=True)
+                        logging.info("Badly formed yEnc article %s", art_id)
 
                     # Continue to the next one if we found new server
                     if search_new_server(article):
@@ -237,7 +239,7 @@ class DecoderWorker(Thread):
 
 def decode_yenc(article: Article, raw_data: List[bytes]) -> bytes:
     # Let SABYenc do all the heavy lifting
-    decoded_data, yenc_filename, crc, crc_expected, crc_correct = sabyenc3.decode_usenet_chunks(raw_data, article.bytes)
+    decoded_data, yenc_filename, crc_correct = sabyenc3.decode_usenet_chunks(raw_data)
 
     # Mark as decoded
     article.decoded = True
