@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2020 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,7 +18,8 @@
 ##############################################################################
 # Decorators
 ##############################################################################
-from threading import RLock, Condition
+from typing import Union, Callable
+from threading import Lock, RLock, Condition
 
 
 # All operations that modify the queue need to happen in a lock
@@ -29,18 +30,22 @@ NZBQUEUE_LOCK = RLock()
 DOWNLOADER_CV = Condition(NZBQUEUE_LOCK)
 
 
-def synchronized(lock):
-    def wrap(f):
+def synchronized(lock: Union[Lock, RLock]):
+    def wrap(func: Callable):
         def call_func(*args, **kw):
-            with lock:
-                return f(*args, **kw)
+            # Using the try/finally approach is 25% faster compared to using "with lock"
+            try:
+                lock.acquire()
+                return func(*args, **kw)
+            finally:
+                lock.release()
 
         return call_func
 
     return wrap
 
 
-def NzbQueueLocker(func):
+def NzbQueueLocker(func: Callable):
     global DOWNLOADER_CV
 
     def call_func(*params, **kparams):
