@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2020 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -32,7 +32,7 @@ import sabnzbd.cfg as cfg
 
 
 def compare_stat_tuple(tup1, tup2):
-    """ Test equality of two stat-tuples, content-related parts only """
+    """Test equality of two stat-tuples, content-related parts only"""
     if tup1.st_ino != tup2.st_ino:
         return False
     if tup1.st_size != tup2.st_size:
@@ -45,8 +45,8 @@ def compare_stat_tuple(tup1, tup2):
 
 
 def clean_file_list(inp_list, folder, files):
-    """ Remove elements of "inp_list" not found in "files" """
-    for path in sorted(inp_list.keys()):
+    """Remove elements of "inp_list" not found in "files" """
+    for path in sorted(inp_list):
         fld, name = os.path.split(path)
         if fld == folder:
             present = False
@@ -65,14 +65,12 @@ class DirScanner(threading.Thread):
     subsequent scans, unless changed.
     """
 
-    do = None  # Access to instance of DirScanner
-
     def __init__(self):
-        threading.Thread.__init__(self)
+        super().__init__()
 
         self.newdir()
         try:
-            dirscan_dir, self.ignored, self.suspected = sabnzbd.load_admin(SCAN_FILE_NAME)
+            dirscan_dir, self.ignored, self.suspected = sabnzbd.filesystem.load_admin(SCAN_FILE_NAME)
             if dirscan_dir != self.dirscan_dir:
                 self.ignored = {}
                 self.suspected = {}
@@ -81,55 +79,53 @@ class DirScanner(threading.Thread):
             # successfully processed ones that cannot be deleted
             self.suspected = {}  # Will hold name/attributes of suspected candidates
 
+        self.loop_condition = threading.Condition(threading.Lock())
         self.shutdown = False
-        self.error_reported = False  # Prevents mulitple reporting of missing watched folder
+        self.error_reported = False  # Prevents multiple reporting of missing watched folder
         self.dirscan_dir = cfg.dirscan_dir.get_path()
-        self.dirscan_speed = cfg.dirscan_speed()
+        self.dirscan_speed = cfg.dirscan_speed() or None  # If set to 0, use None so the wait() is forever
         self.busy = False
-        self.trigger = False
         cfg.dirscan_dir.callback(self.newdir)
         cfg.dirscan_speed.callback(self.newspeed)
-        DirScanner.do = self
 
     def newdir(self):
-        """ We're notified of a dir change """
+        """We're notified of a dir change"""
         self.ignored = {}
         self.suspected = {}
         self.dirscan_dir = cfg.dirscan_dir.get_path()
         self.dirscan_speed = cfg.dirscan_speed()
 
     def newspeed(self):
-        """ We're notified of a scan speed change """
-        self.dirscan_speed = cfg.dirscan_speed()
-        self.trigger = True
+        """We're notified of a scan speed change"""
+        # If set to 0, use None so the wait() is forever
+        self.dirscan_speed = cfg.dirscan_speed() or None
+        with self.loop_condition:
+            self.loop_condition.notify()
 
     def stop(self):
-        """ Stop the dir scanner """
-        logging.info("Dirscanner shutting down")
+        """Stop the dir scanner"""
         self.shutdown = True
+        with self.loop_condition:
+            self.loop_condition.notify()
 
     def save(self):
-        """ Save dir scanner bookkeeping """
-        sabnzbd.save_admin((self.dirscan_dir, self.ignored, self.suspected), SCAN_FILE_NAME)
+        """Save dir scanner bookkeeping"""
+        sabnzbd.filesystem.save_admin((self.dirscan_dir, self.ignored, self.suspected), SCAN_FILE_NAME)
 
     def run(self):
-        """ Start the scanner """
+        """Start the scanner"""
         logging.info("Dirscanner starting up")
         self.shutdown = False
 
         while not self.shutdown:
-            # Use variable scan delay
-            x = max(self.dirscan_speed, 1)
-            while (x > 0) and not self.shutdown and not self.trigger:
-                time.sleep(1.0)
-                x = x - 1
-
-            self.trigger = False
+            # Wait to be woken up or triggered
+            with self.loop_condition:
+                self.loop_condition.wait(self.dirscan_speed)
             if self.dirscan_speed and not self.shutdown:
                 self.scan()
 
     def scan(self):
-        """ Do one scan of the watched folder """
+        """Do one scan of the watched folder"""
 
         def run_dir(folder, catdir):
             try:
@@ -141,6 +137,8 @@ class DirScanner(threading.Thread):
                 files = []
 
             for filename in files:
+                if self.shutdown:
+                    break
                 path = os.path.join(folder, filename)
                 if os.path.isdir(path) or path in self.ignored or filename[0] == ".":
                     continue
@@ -180,7 +178,7 @@ class DirScanner(threading.Thread):
                         continue
 
                     # Add the NZB's
-                    res, _ = sabnzbd.add_nzbfile(path, catdir=catdir, keep=False)
+                    res, _ = sabnzbd.nzbparser.add_nzbfile(path, catdir=catdir, keep=False)
                     if res < 0:
                         # Retry later, for example when we can't read the file
                         self.suspected[path] = stat_tuple
@@ -213,9 +211,3 @@ class DirScanner(threading.Thread):
                     if os.path.isdir(dpath) and dd.lower() in cats:
                         run_dir(dpath, dd.lower())
             self.busy = False
-
-
-def dirscan():
-    """ Wrapper required for scheduler """
-    logging.info("Scheduled or manual watched folder scan")
-    DirScanner.do.scan()
