@@ -24,7 +24,6 @@ import select
 import logging
 from math import ceil
 from threading import Thread, RLock
-from nntplib import NNTPPermanentError
 import socket
 import random
 import sys
@@ -33,10 +32,10 @@ from typing import List, Dict, Optional, Union
 
 import sabnzbd
 from sabnzbd.decorators import synchronized, NzbQueueLocker, DOWNLOADER_CV
-from sabnzbd.newswrapper import NewsWrapper
+from sabnzbd.newswrapper import NewsWrapper, NNTPPermanentError
 import sabnzbd.config as config
 import sabnzbd.cfg as cfg
-from sabnzbd.misc import from_units, nntp_to_msg, int_conv, get_server_addrinfo, helpful_warning
+from sabnzbd.misc import from_units, nntp_to_msg, get_server_addrinfo, helpful_warning, int_conv
 from sabnzbd.utils.happyeyeballs import happyeyeballs
 
 
@@ -54,6 +53,8 @@ _PENALTY_VERYSHORT = 0.1  # Error 400 without cause clues
 _SERVER_CHECK_DELAY = 0.5
 # Wait this many seconds between updates of the BPSMeter
 _BPSMETER_UPDATE_DELAY = 0.05
+# How many articles should be prefetched when checking the next articles?
+_ARTICLE_PREFETCH = 20
 
 TIMER_LOCK = RLock()
 
@@ -152,12 +153,14 @@ class Server:
         self.have_stat: bool = True  # Assume server has "STAT", until proven otherwise
         self.article_queue: List[sabnzbd.nzbstuff.Article] = []
 
-        # Initialize threads
-        for i in range(threads):
-            self.idle_threads.append(NewsWrapper(self, i + 1))
+        # Skip during server testing
+        if threads:
+            # Initialize threads
+            for i in range(threads):
+                self.idle_threads.append(NewsWrapper(self, i + 1))
 
-        # Tell the BPSMeter about this server
-        sabnzbd.BPSMeter.init_server_stats(self.id)
+            # Tell the BPSMeter about this server
+            sabnzbd.BPSMeter.init_server_stats(self.id)
 
     @property
     def hostip(self) -> str:
@@ -177,27 +180,25 @@ class Server:
         # Determine IP
         ip = self.host
         if self.info:
-            if cfg.load_balancing() == 0 or len(self.info) == 1:
+            # Check this first so we can fall back to default method if it returns None.
+            if len(self.info) > 1 and cfg.load_balancing() == 2:
+                # RFC6555 / Happy Eyeballs:
+                ip = happyeyeballs(self.host, port=self.port)
+                if not ip:
+                    # nothing returned, so there was a connection problem
+                    logging.debug("%s: No successful IP connection was possible using happyeyeballs", self.host)
+            if not ip or cfg.load_balancing() == 0 or len(self.info) == 1:
                 # Just return the first one, so all next threads use the same IP
                 ip = self.info[0][4][0]
-                logging.debug("%s: Connecting to address %s", self.host, ip)
             elif cfg.load_balancing() == 1:
                 # Return a random entry from the possible IPs
                 rnd = random.randint(0, len(self.info) - 1)
                 ip = self.info[rnd][4][0]
-                logging.debug("%s: Connecting to address %s", self.host, ip)
-            elif cfg.load_balancing() == 2:
-                # RFC6555 / Happy Eyeballs:
-                ip = happyeyeballs(self.host, port=self.port)
-                if ip:
-                    logging.debug("%s: Connecting to address %s", self.host, ip)
-                else:
-                    # nothing returned, so there was a connection problem
-                    logging.debug("%s: No successful IP connection was possible", self.host)
+        logging.debug("%s: Connecting to address %s", self.host, ip)
         return ip
 
     def deactivate(self):
-        """Deactive server and reset queued articles"""
+        """Deactivate server and reset queued articles"""
         self.active = False
         self.reset_article_queue()
 
@@ -229,6 +230,7 @@ class Server:
         self.info = get_server_addrinfo(self.host, self.port)
         if not self.info:
             self.bad_cons += self.threads
+            self.info = False
         else:
             self.bad_cons = 0
         self.request = False
@@ -265,7 +267,7 @@ class Downloader(Thread):
         # Used for scheduled pausing
         self.paused: bool = paused
 
-        # Used for reducing speed
+        # Used for reducing speed, should always be int and not float
         self.bandwidth_limit: int = 0
         self.bandwidth_perc: int = 0
         cfg.bandwidth_perc.callback(self.speed_set)
@@ -395,7 +397,7 @@ class Downloader(Thread):
             sabnzbd.notifier.send_notification("SABnzbd", T("Paused"), "pause_resume")
             if cfg.preserve_paused_state():
                 cfg.start_paused.set(True)
-            if self.is_paused():
+            if self.no_active_jobs():
                 sabnzbd.BPSMeter.reset()
             if cfg.autodisconnect():
                 self.disconnect()
@@ -416,40 +418,34 @@ class Downloader(Thread):
 
     def limit_speed(self, value: Union[str, int]):
         """Set the actual download speed in Bytes/sec
-        When 'value' ends with a '%' sign or is within 1-100, it is interpreted as a pecentage of the maximum bandwidth
+        When 'value' ends with a '%' sign or is within 1-100, it is interpreted as a percentage of the maximum bandwidth
         When no '%' is found, it is interpreted as an absolute speed (including KMGT notation).
         """
         if value:
             mx = cfg.bandwidth_max.get_int()
             if "%" in str(value) or (0 < from_units(value) < 101):
                 limit = value.strip(" %")
-                self.bandwidth_perc = from_units(limit)
+                self.bandwidth_perc = int_conv(limit)
                 if mx:
-                    self.bandwidth_limit = mx * self.bandwidth_perc / 100
+                    self.bandwidth_limit = int(mx * self.bandwidth_perc / 100)
                 else:
                     helpful_warning(T("You must set a maximum bandwidth before you can set a bandwidth limit"))
             else:
-                self.bandwidth_limit = from_units(value)
+                self.bandwidth_limit = int(from_units(value))
                 if mx:
-                    self.bandwidth_perc = self.bandwidth_limit / mx * 100
+                    self.bandwidth_perc = int(self.bandwidth_limit / mx * 100)
                 else:
                     self.bandwidth_perc = 100
         else:
             self.speed_set()
         logging.info("Speed limit set to %s B/s", self.bandwidth_limit)
 
-    def get_limit(self):
-        return self.bandwidth_perc
-
-    def get_limit_abs(self):
-        return self.bandwidth_limit
-
     def speed_set(self):
-        limit = cfg.bandwidth_max.get_int()
         perc = cfg.bandwidth_perc()
+        limit = cfg.bandwidth_max.get_int()
         if limit and perc:
-            self.bandwidth_perc = perc
-            self.bandwidth_limit = limit * perc / 100
+            self.bandwidth_perc = int(perc)
+            self.bandwidth_limit = int(limit * perc / 100)
         else:
             self.bandwidth_perc = 0
             self.bandwidth_limit = 0
@@ -458,26 +454,19 @@ class Downloader(Thread):
         self.sleep_time = cfg.downloader_sleep_time() * 0.0001
         logging.debug("Sleep time: %f seconds", self.sleep_time)
 
-    def is_paused(self):
-        if not self.paused:
-            return False
-        else:
-            if sabnzbd.NzbQueue.has_forced_items():
-                return False
-            else:
-                return True
+    def no_active_jobs(self) -> bool:
+        """Is the queue paused or is it paused but are there still forced items?"""
+        return self.paused and not sabnzbd.NzbQueue.has_forced_jobs()
 
     def highest_server(self, me: Server):
         """Return True when this server has the highest priority of the active ones
-        0 is the highest priority
+        0 is the highest priority, servers are sorted by priority.
         """
         for server in self.servers:
-            if server is not me and server.active and server.priority < me.priority:
+            if server.priority == me.priority:
+                return True
+            if server.active:
                 return False
-        return True
-
-    def nzo_servers(self, nzo):
-        return list(filter(nzo.server_in_try_list, self.servers))
 
     def maybe_block_server(self, server: Server):
         # Was it resolving problem?
@@ -514,7 +503,7 @@ class Downloader(Thread):
             # Make sure server address resolution is refreshed
             server.info = None
 
-    def decode(self, article, raw_data: Optional[List[bytes]]):
+    def decode(self, article, raw_data: Optional[List[bytes]] = None, data_size: Optional[int] = None):
         """Decode article and check the status of
         the decoder and the assembler
         """
@@ -529,20 +518,33 @@ class Downloader(Thread):
             return
 
         # Send to decoder-queue
-        sabnzbd.Decoder.process(article, raw_data)
+        sabnzbd.Decoder.process(article, raw_data, data_size)
 
         # See if we need to delay because the queues are full
-        logged = False
-        while not self.shutdown and (sabnzbd.Decoder.queue_full() or sabnzbd.Assembler.queue_full()):
-            if not logged:
-                # Only log once, to not waste any CPU-cycles
+        logged_counter = 0
+        decoder_full = sabnzbd.Decoder.queue_full()
+        assembler_full = sabnzbd.Assembler.queue_full()
+        while not self.shutdown and (decoder_full or assembler_full):
+            # Only log/update once every second, to not waste any CPU-cycles
+            if not logged_counter % 10:
+                # Make sure the BPS-meter is updated
+                sabnzbd.BPSMeter.update()
+
+                # Update who is delaying us
+                sabnzbd.BPSMeter.delayed_decoder += int(decoder_full)
+                sabnzbd.BPSMeter.delayed_assembler += int(assembler_full)
                 logging.debug(
-                    "Delaying - Decoder queue: %s - Assembler queue: %s",
+                    "Delayed - %d seconds - Decoder queue: %d - Assembler queue: %d",
+                    logged_counter / 10,
                     sabnzbd.Decoder.decoder_queue.qsize(),
                     sabnzbd.Assembler.queue.qsize(),
                 )
-                logged = True
-            time.sleep(0.01)
+
+            # Wait and update the queue sizes
+            time.sleep(0.1)
+            logged_counter += 1
+            decoder_full = sabnzbd.Decoder.queue_full()
+            assembler_full = sabnzbd.Assembler.queue_full()
 
     def run(self):
         # First check IPv6 connectivity
@@ -553,6 +555,10 @@ class Downloader(Thread):
         sabnzbd.CERTIFICATE_VALIDATION = sabnzbd.misc.test_cert_checking()
         logging.debug("SSL verification test: %s", sabnzbd.CERTIFICATE_VALIDATION)
 
+        # Warn if there are servers defined, but none are valid
+        if config.get_servers() and not self.servers:
+            logging.warning(T("There are no active servers!"))
+
         # Kick BPS-Meter to check quota
         BPSMeter = sabnzbd.BPSMeter
         BPSMeter.update()
@@ -606,7 +612,7 @@ class Downloader(Thread):
 
                 if (
                     not server.idle_threads
-                    or self.is_paused()
+                    or self.no_active_jobs()
                     or self.shutdown
                     or self.paused_for_postproc
                     or not server.active
@@ -632,16 +638,14 @@ class Downloader(Thread):
                         article = server.article_queue.pop(0)
                     else:
                         # Pre-fetch new articles
-                        server.article_queue = sabnzbd.NzbQueue.get_articles(
-                            server, self.servers, max(1, server.threads // 4)
-                        )
+                        server.article_queue = sabnzbd.NzbQueue.get_articles(server, self.servers, _ARTICLE_PREFETCH)
                         if server.article_queue:
                             article = server.article_queue.pop(0)
                             # Mark expired articles as tried on this server
                             if server.retention and article.nzf.nzo.avg_stamp < now - server.retention:
-                                self.decode(article, None)
+                                self.decode(article)
                                 while server.article_queue:
-                                    self.decode(server.article_queue.pop(), None)
+                                    self.decode(server.article_queue.pop())
                                 # Move to the next server, allowing the next server to already start
                                 # fetching the articles that were too old for this server
                                 break
@@ -730,7 +734,7 @@ class Downloader(Thread):
 
                 with DOWNLOADER_CV:
                     while (
-                        (sabnzbd.NzbQueue.is_empty() or self.is_paused() or self.paused_for_postproc)
+                        (sabnzbd.NzbQueue.is_empty() or self.no_active_jobs() or self.paused_for_postproc)
                         and not self.shutdown
                         and not self.force_disconnect
                         and not self.server_restarts
@@ -760,14 +764,7 @@ class Downloader(Thread):
                 if bytes_received < 1:
                     self.__reset_nw(nw, "server closed connection", wait=False)
                     continue
-
                 else:
-                    try:
-                        article.nzf.nzo.update_download_stats(BPSMeter.bps, server.id, bytes_received)
-                    except AttributeError:
-                        # In case nzf has disappeared because the file was deleted before the update could happen
-                        pass
-
                     BPSMeter.update(server.id, bytes_received)
 
                     if self.bandwidth_limit:
@@ -790,11 +787,9 @@ class Downloader(Thread):
                             # Handle login problems
                             block = False
                             penalty = 0
-                            msg = error.response
-                            ecode = int_conv(msg[:3])
-                            display_msg = " [%s]" % msg
-                            logging.debug("Server login problem: %s, %s", ecode, msg)
-                            if ecode in (502, 400, 481, 482) and clues_too_many(msg):
+                            display_msg = " [%s]" % error.msg
+                            logging.debug("Server login problem: %s", error.msg)
+                            if error.code in (502, 400, 481, 482) and clues_too_many(error.msg):
                                 # Too many connections: remove this thread and reduce thread-setting for server
                                 # Plan to go back to the full number after a penalty timeout
                                 if server.active:
@@ -806,7 +801,7 @@ class Downloader(Thread):
                                     self.__reset_nw(nw, send_quit=True)
                                     self.plan_server(server, _PENALTY_TOOMANY)
                                     server.threads -= 1
-                            elif ecode in (502, 481, 482) and clues_too_many_ip(msg):
+                            elif error.code in (502, 481, 482) and clues_too_many_ip(error.msg):
                                 # Account sharing?
                                 if server.active:
                                     errormsg = T("Probable account sharing") + display_msg
@@ -816,7 +811,9 @@ class Downloader(Thread):
                                         logging.warning(T("Probable account sharing") + name)
                                 penalty = _PENALTY_SHARE
                                 block = True
-                            elif ecode in (452, 481, 482, 381) or (ecode == 502 and clues_login(msg)):
+                            elif error.code in (452, 481, 482, 381) or (
+                                error.code in (500, 502) and clues_login(error.msg)
+                            ):
                                 # Cannot login, block this server
                                 if server.active:
                                     errormsg = T("Failed login for server %s") % display_msg
@@ -825,19 +822,19 @@ class Downloader(Thread):
                                         logging.error(T("Failed login for server %s"), server.host)
                                 penalty = _PENALTY_PERM
                                 block = True
-                            elif ecode in (502, 482):
+                            elif error.code in (502, 482):
                                 # Cannot connect (other reasons), block this server
                                 if server.active:
-                                    errormsg = T("Cannot connect to server %s [%s]") % ("", display_msg)
+                                    errormsg = T("Cannot connect to server %s [%s]") % ("", error.msg)
                                     if server.errormsg != errormsg:
                                         server.errormsg = errormsg
-                                        logging.warning(T("Cannot connect to server %s [%s]"), server.host, msg)
-                                if clues_pay(msg):
+                                        logging.warning(T("Cannot connect to server %s [%s]"), server.host, error.msg)
+                                if clues_pay(error.msg):
                                     penalty = _PENALTY_PERM
                                 else:
                                     penalty = _PENALTY_502
                                 block = True
-                            elif ecode == 400:
+                            elif error.code == 400:
                                 # Temp connection problem?
                                 if server.active:
                                     logging.debug("Unspecified error 400 from server %s", server.host)
@@ -849,7 +846,7 @@ class Downloader(Thread):
                                     errormsg = T("Cannot connect to server %s [%s]") % ("", display_msg)
                                     if server.errormsg != errormsg:
                                         server.errormsg = errormsg
-                                        logging.warning(T("Cannot connect to server %s [%s]"), server.host, msg)
+                                        logging.warning(T("Cannot connect to server %s [%s]"), server.host, error.msg)
                                 penalty = _PENALTY_UNKNOWN
                                 block = True
                             if block or (penalty and server.optional):
@@ -916,9 +913,13 @@ class Downloader(Thread):
                     # Successful data, clear "bad" counter
                     server.bad_cons = 0
                     server.errormsg = server.warning = ""
+
+                    # Update statistics and decode
+                    article.nzf.nzo.update_download_stats(BPSMeter.bps, server.id, nw.data_size)
+                    self.decode(article, nw.data, nw.data_size)
+
                     if sabnzbd.LOG_ALL:
                         logging.debug("Thread %s@%s: %s done", nw.thrdnum, server.host, article.article)
-                    self.decode(article, nw.data)
 
                     # Reset connection for new activity
                     nw.soft_reset()
@@ -958,9 +959,9 @@ class Downloader(Thread):
                 nw.article.tries += 1
 
             # Do we discard, or try again for this server
-            if not retry_article or nw.article.tries > cfg.max_art_tries():
+            if not retry_article or (not nw.server.required and nw.article.tries > cfg.max_art_tries()):
                 # Too many tries on this server, consider article missing
-                self.decode(nw.article, None)
+                self.decode(nw.article)
                 nw.article.tries = 0
             else:
                 # Retry again with the same server
