@@ -68,8 +68,9 @@ import sabnzbd.cfg as cfg
 import sabnzbd.notifier as notifier
 import sabnzbd.newsunpack
 from sabnzbd.utils.servertests import test_nntp_server_dict
+from sabnzbd.utils.getperformance import getcpu
 import sabnzbd.utils.ssdp
-from sabnzbd.constants import DEF_STDCONFIG, DEFAULT_PRIORITY, CHEETAH_DIRECTIVES, EXCLUDED_GUESSIT_PROPERTIES
+from sabnzbd.constants import DEF_STD_CONFIG, DEFAULT_PRIORITY, CHEETAH_DIRECTIVES, EXCLUDED_GUESSIT_PROPERTIES
 from sabnzbd.lang import list_languages
 from sabnzbd.api import (
     list_scripts,
@@ -144,12 +145,16 @@ def secured_expose(
         # Check if config is locked
         if check_configlock and cfg.configlock():
             cherrypy.response.status = 403
-            return _MSG_ACCESS_DENIED_CONFIG_LOCK
+            if cfg.api_warnings():
+                return _MSG_ACCESS_DENIED_CONFIG_LOCK
+            return
 
         # Check if external access and if it's allowed
         if not check_access(access_type=access_type, warn_user=True):
             cherrypy.response.status = 403
-            return _MSG_ACCESS_DENIED
+            if cfg.api_warnings():
+                return _MSG_ACCESS_DENIED
+            return
 
         # Verify login status, only for non-key pages
         if check_for_login and not check_api_key and not check_login():
@@ -158,14 +163,18 @@ def secured_expose(
         # Verify host used for the visit
         if not check_hostname():
             cherrypy.response.status = 403
-            return _MSG_ACCESS_DENIED_HOSTNAME
+            if cfg.api_warnings():
+                return _MSG_ACCESS_DENIED_HOSTNAME
+            return
 
         # Some pages need correct API key
         if check_api_key:
             msg = check_apikey(kwargs)
             if msg:
                 cherrypy.response.status = 403
-                return msg
+                if cfg.api_warnings():
+                    return msg
+                return
 
         # All good, cool!
         return wrap_func(*args, **kwargs)
@@ -335,35 +344,19 @@ def check_apikey(kwargs):
         return None
 
     # First check API-key, if OK that's sufficient
-    if not cfg.disable_key():
-        key = kwargs.get("apikey")
-        if not key:
-            log_warning_and_ip(
-                T("API Key missing, please enter the api key from Config->General into your 3rd party program:")
-            )
-            return _MSG_APIKEY_REQUIRED
-        elif req_access == 1 and key == cfg.nzb_key():
-            return None
-        elif key == cfg.api_key():
-            return None
-        else:
-            log_warning_and_ip(T("API Key incorrect, Use the api key from Config->General in your 3rd party program:"))
-            return _MSG_APIKEY_INCORRECT
-
-    # No active API-key, check web credentials instead
-    if cfg.username() and cfg.password():
-        if check_login() or (
-            kwargs.get("ma_username") == cfg.username() and kwargs.get("ma_password") == cfg.password()
-        ):
-            pass
-        else:
-            log_warning_and_ip(
-                T(
-                    "Authentication missing, please enter username/password from Config->General into your 3rd party program:"
-                )
-            )
-            return _MSG_MISSING_AUTH
-    return None
+    key = kwargs.get("apikey")
+    if not key:
+        log_warning_and_ip(
+            T("API Key missing, please enter the api key from Config->General into your 3rd party program:")
+        )
+        return _MSG_APIKEY_REQUIRED
+    elif req_access == 1 and key == cfg.nzb_key():
+        return None
+    elif key == cfg.api_key():
+        return None
+    else:
+        log_warning_and_ip(T("API Key incorrect, Use the api key from Config->General in your 3rd party program:"))
+        return _MSG_APIKEY_INCORRECT
 
 
 def template_filtered_response(file: str, search_list: Dict[str, Any]):
@@ -426,6 +419,9 @@ class MainPage:
             info["have_rss_defined"] = bool(config.get_rss())
             info["have_watched_dir"] = bool(cfg.dirscan_dir())
 
+            info["cpumodel"] = getcpu()
+            info["cpusimd"] = sabnzbd.decoder.SABYENC_SIMD
+
             # Have logout only with HTML and if inet=5, only when we are external
             info["have_logout"] = (
                 cfg.username()
@@ -511,15 +507,13 @@ class Wizard:
         if kwargs.get("lang"):
             cfg.language.set(kwargs.get("lang"))
 
-        # Always setup Glitter
-        change_web_dir("Glitter - Auto")
-
         info = build_header(sabnzbd.WIZARD_DIR)
         info["certificate_validation"] = sabnzbd.CERTIFICATE_VALIDATION
 
         # Just in case, add server
         servers = config.get_servers()
         if not servers:
+            info["server"] = ""
             info["host"] = ""
             info["port"] = ""
             info["username"] = ""
@@ -537,6 +531,7 @@ class Wizard:
             for server in server_names:
                 # If there are multiple servers, just use the first enabled one
                 s = servers[server]
+                info["server"] = server
                 info["host"] = s.host()
                 info["port"] = s.port()
                 info["username"] = s.username()
@@ -573,14 +568,16 @@ def get_access_info():
     # Access_url is used to provide the user a link to SABnzbd depending on the host
     cherryhost = cfg.cherryhost()
     host = socket.gethostname().lower()
+    logging.info("hostname is", host)
     socks = [host]
 
+    try:
+        addresses = socket.getaddrinfo(host, None)
+    except:
+        addresses = []
+
     if cherryhost == "0.0.0.0":
         # Grab a list of all ips for the hostname
-        try:
-            addresses = socket.getaddrinfo(host, None)
-        except:
-            addresses = []
         for addr in addresses:
             address = addr[4][0]
             # Filter out ipv6 addresses (should not be allowed)
@@ -589,7 +586,6 @@ def get_access_info():
         socks.insert(0, "localhost")
     elif cherryhost == "::":
         # Grab a list of all ips for the hostname
-        addresses = socket.getaddrinfo(host, None)
         for addr in addresses:
             address = addr[4][0]
             # Only ipv6 addresses will work
@@ -706,6 +702,7 @@ LIST_DIRPAGE = (
     "email_dir",
     "permissions",
     "log_dir",
+    "backup_dir",
     "password_file",
 )
 
@@ -733,7 +730,7 @@ class ConfigFolders:
         for kw in LIST_DIRPAGE + LIST_BOOL_DIRPAGE:
             value = kwargs.get(kw)
             if value is not None or kw in LIST_BOOL_DIRPAGE:
-                if kw in ("complete_dir", "dirscan_dir"):
+                if kw in ("complete_dir", "dirscan_dir", "backup_dir"):
                     msg = config.get_config("misc", kw).set(value, create=True)
                 else:
                     msg = config.get_config("misc", kw).set(value)
@@ -758,7 +755,6 @@ SWITCH_LIST = (
     "par_option",
     "top_only",
     "direct_unpack",
-    "enable_meta",
     "win_process_prio",
     "auto_sort",
     "propagation_delay",
@@ -766,6 +762,7 @@ SWITCH_LIST = (
     "flat_unpack",
     "safe_postproc",
     "no_dupes",
+    "replace_underscores",
     "replace_spaces",
     "replace_dots",
     "ignore_samples",
@@ -795,26 +792,8 @@ SWITCH_LIST = (
     "unwanted_extensions",
     "action_on_unwanted_extensions",
     "unwanted_extensions_mode",
+    "cleanup_list",
     "sanitize_safe",
-    "rating_enable",
-    "rating_api_key",
-    "rating_filter_enable",
-    "rating_filter_abort_audio",
-    "rating_filter_abort_video",
-    "rating_filter_abort_encrypted",
-    "rating_filter_abort_encrypted_confirm",
-    "rating_filter_abort_spam",
-    "rating_filter_abort_spam_confirm",
-    "rating_filter_abort_downvoted",
-    "rating_filter_abort_keywords",
-    "rating_filter_pause_audio",
-    "rating_filter_pause_video",
-    "rating_filter_pause_encrypted",
-    "rating_filter_pause_encrypted_confirm",
-    "rating_filter_pause_spam",
-    "rating_filter_pause_spam_confirm",
-    "rating_filter_pause_downvoted",
-    "rating_filter_pause_keywords",
 )
 
 
@@ -829,10 +808,10 @@ class ConfigSwitches:
         conf["certificate_validation"] = sabnzbd.CERTIFICATE_VALIDATION
         conf["have_nice"] = bool(sabnzbd.newsunpack.NICE_COMMAND)
         conf["have_ionice"] = bool(sabnzbd.newsunpack.IONICE_COMMAND)
-        conf["cleanup_list"] = cfg.cleanup_list.get_string()
 
         for kw in SWITCH_LIST:
             conf[kw] = config.get_config("misc", kw)()
+        conf["cleanup_list"] = cfg.cleanup_list.get_string()
         conf["unwanted_extensions"] = cfg.unwanted_extensions.get_string()
 
         conf["scripts"] = list_scripts() or ["None"]
@@ -845,19 +824,10 @@ class ConfigSwitches:
     @secured_expose(check_api_key=True, check_configlock=True)
     def saveSwitches(self, **kwargs):
         for kw in SWITCH_LIST:
-            item = config.get_config("misc", kw)
-            value = kwargs.get(kw)
-            if kw == "unwanted_extensions" and value:
-                value = value.lower().replace(".", "")
-            msg = item.set(value)
+            msg = config.get_config("misc", kw).set(kwargs.get(kw))
             if msg:
                 return badParameterResponse(msg, kwargs.get("ajax"))
 
-        cleanup_list = kwargs.get("cleanup_list")
-        if cleanup_list and sabnzbd.WIN32:
-            cleanup_list = cleanup_list.lower()
-        cfg.cleanup_list.set(cleanup_list)
-
         config.save_config()
         if kwargs.get("ajax"):
             return sabnzbd.api.report()
@@ -884,9 +854,7 @@ SPECIAL_BOOL_LIST = (
     "enable_filejoin",
     "enable_tsjoin",
     "ignore_unrar_dates",
-    "osx_menu",
-    "osx_speed",
-    "win_menu",
+    "tray_icon",
     "allow_incomplete_nzb",
     "rss_filenames",
     "ipv6_hosting",
@@ -896,9 +864,7 @@ SPECIAL_BOOL_LIST = (
     "wait_for_dfolder",
     "enable_broadcast",
     "warn_dupl_jobs",
-    "replace_illegal",
     "backup_for_duplicates",
-    "disable_api_key",
     "api_logging",
     "x_frame_options",
     "allow_old_ssl_tls",
@@ -913,13 +879,11 @@ SPECIAL_VALUE_LIST = (
     "req_completion_rate",
     "wait_ext_drive",
     "max_foldername_length",
-    "show_sysload",
     "url_base",
-    "num_decoders",
+    "num_simd_decoders",
     "direct_unpack_threads",
     "ipv6_servers",
     "selftest_host",
-    "rating_host",
     "ssdp_broadcast_interval",
 )
 SPECIAL_LIST_LIST = (
@@ -927,6 +891,7 @@ SPECIAL_LIST_LIST = (
     "quick_check_ext_ignore",
     "host_whitelist",
     "local_ranges",
+    "ext_rename_ignore",
 )
 
 
@@ -938,10 +903,10 @@ class ConfigSpecial:
     def index(self, **kwargs):
         conf = build_header(sabnzbd.WEB_DIR_CONFIG)
         conf["switches"] = [
-            (kw, config.get_config("misc", kw)(), config.get_config("misc", kw).default()) for kw in SPECIAL_BOOL_LIST
+            (kw, config.get_config("misc", kw)(), config.get_config("misc", kw).default) for kw in SPECIAL_BOOL_LIST
         ]
         conf["entries"] = [
-            (kw, config.get_config("misc", kw)(), config.get_config("misc", kw).default()) for kw in SPECIAL_VALUE_LIST
+            (kw, config.get_config("misc", kw)(), config.get_config("misc", kw).default) for kw in SPECIAL_VALUE_LIST
         ]
         conf["entries"].extend(
             [
@@ -973,7 +938,6 @@ GENERAL_LIST = (
     "host",
     "port",
     "username",
-    "refresh_rate",
     "language",
     "cache_limit",
     "inet_exposure",
@@ -998,13 +962,12 @@ class ConfigGeneral:
 
         conf = build_header(sabnzbd.WEB_DIR_CONFIG)
 
-        conf["configfn"] = config.get_filename()
         conf["certificate_validation"] = sabnzbd.CERTIFICATE_VALIDATION
 
         web_list = []
         for interface_dir in globber_full(sabnzbd.DIR_INTERFACES):
             # Ignore the config
-            if not interface_dir.endswith(DEF_STDCONFIG):
+            if not interface_dir.endswith(DEF_STD_CONFIG):
                 # Check the available templates
                 for colorscheme in globber(
                     os.path.join(interface_dir, "templates", "static", "stylesheets", "colorschemes")
@@ -1039,7 +1002,7 @@ class ConfigGeneral:
             value = kwargs.get(kw)
             msg = item.set(value)
             if msg:
-                return badParameterResponse(msg)
+                return badParameterResponse(msg, ajax=kwargs.get("ajax"))
 
         # Handle special options
         cfg.password.set(kwargs.get("password"))
@@ -1066,6 +1029,22 @@ class ConfigGeneral:
         else:
             raise Raiser(self.__root)
 
+    @secured_expose(check_api_key=True, check_configlock=True)
+    def uploadConfig(self, **kwargs):
+        """Restore a config backup"""
+        config_backup_file = kwargs.get("config_backup_file")
+
+        # Only accept the backup file if it can be opened as a zip archive and only contains a config file
+        try:
+            config_backup_data = config_backup_file.file.read()
+            config_backup_file.file.close()
+            if config.validate_config_backup(config_backup_data):
+                sabnzbd.RESTORE_DATA = config_backup_data
+                return sabnzbd.api.report(data={"success": True, "restart_req": True})
+        except:
+            pass
+        return sabnzbd.api.report(error=T("Invalid backup archive"))
+
 
 def change_web_dir(web_dir):
     web_dir, web_color = web_dir.split(" - ")
@@ -1374,9 +1353,8 @@ class ConfigRss:
             # Did we get a new name for this feed?
             new_name = kwargs.get("feed_new_name")
             if new_name and new_name != feed_name:
-                cf.rename(new_name)
                 # Update the feed name for the redirect
-                kwargs["feed"] = new_name
+                kwargs["feed"] = cf.rename(new_name)
 
             config.save_config()
 
@@ -1589,6 +1567,7 @@ _SCHED_ACTIONS = (
     "resume_post",
     "scan_folder",
     "rss_scan",
+    "create_backup",
     "remove_failed",
     "remove_completed",
     "pause_all_low",
@@ -1819,10 +1798,10 @@ class ConfigCats:
     @secured_expose(check_api_key=True, check_configlock=True)
     def save(self, **kwargs):
         name = kwargs.get("name", "*")
+        newname = kwargs.get("newname", "")
         if name == "*":
             newname = name
-        else:
-            newname = re.sub('"', "", kwargs.get("newname", ""))
+
         if newname:
             # Check if this cat-dir is not sub-folder of incomplete
             if same_file(cfg.download_dir.get_path(), real_path(cfg.complete_dir.get_path(), kwargs["dir"])):
@@ -1845,7 +1824,6 @@ SORT_LIST = (
     "enable_movie_sorting",
     "movie_sort_string",
     "movie_sort_extra",
-    "movie_extra_folder",
     "enable_date_sorting",
     "date_sort_string",
     "movie_categories",
@@ -2153,7 +2131,7 @@ class ConfigNotify:
         conf["notify_types"] = sabnzbd.notifier.NOTIFICATION
         conf["categories"] = list_cats(False)
         conf["have_ntfosd"] = sabnzbd.notifier.have_ntfosd()
-        conf["have_ncenter"] = sabnzbd.DARWIN and sabnzbd.FOUNDATION
+        conf["have_ncenter"] = sabnzbd.MACOS and sabnzbd.FOUNDATION
         conf["scripts"] = list_scripts(default=False, none=True)
 
         for section in NOTIFY_OPTIONS:
