@@ -22,7 +22,6 @@ sabnzbd.newswrapper
 import errno
 import socket
 from threading import Thread
-from nntplib import NNTPPermanentError
 import time
 import logging
 import ssl
@@ -38,6 +37,16 @@ from sabnzbd.misc import nntp_to_msg, is_ipv4_addr, is_ipv6_addr, get_server_add
 socket.setdefaulttimeout(DEF_TIMEOUT)
 
 
+class NNTPPermanentError(Exception):
+    def __init__(self, msg: str, code: int):
+        super().__init__()
+        self.msg = msg
+        self.code = code
+
+    def __str__(self) -> str:
+        return self.msg
+
+
 class NewsWrapper:
     # Pre-define attributes to save memory
     __slots__ = (
@@ -47,6 +56,7 @@ class NewsWrapper:
         "timeout",
         "article",
         "data",
+        "data_size",
         "nntp",
         "connected",
         "user_sent",
@@ -66,6 +76,7 @@ class NewsWrapper:
         self.timeout: Optional[float] = None
         self.article: Optional[sabnzbd.nzbstuff.Article] = None
         self.data: List[AnyStr] = []
+        self.data_size: int = 0
 
         self.nntp: Optional[NNTP] = None
 
@@ -114,8 +125,8 @@ class NewsWrapper:
             self.pass_sent = False
             self.pass_ok = False
 
-        if code in (400, 502):
-            raise NNTPPermanentError(nntp_to_msg(self.data))
+        if code in (400, 500, 502):
+            raise NNTPPermanentError(nntp_to_msg(self.data), code)
         elif not self.user_sent:
             command = utob("authinfo user %s\r\n" % self.server.username)
             self.nntp.sock.sendall(command)
@@ -139,7 +150,7 @@ class NewsWrapper:
         elif self.user_ok and not self.pass_ok:
             if code != 281:
                 # Assume that login failed (code 481 or other)
-                raise NNTPPermanentError(nntp_to_msg(self.data))
+                raise NNTPPermanentError(nntp_to_msg(self.data), code)
             else:
                 self.connected = True
 
@@ -198,9 +209,10 @@ class NewsWrapper:
 
         # Append so we can do 1 join(), much faster than multiple!
         self.data.append(chunk)
+        chunk_len = len(chunk)
+        self.data_size += chunk_len
 
         # Official end-of-article is ".\r\n" but sometimes it can get lost between 2 chunks
-        chunk_len = len(chunk)
         if chunk[-5:] == b"\r\n.\r\n":
             return chunk_len, True, False
         elif chunk_len < 5 and len(self.data) > 1:
@@ -222,6 +234,7 @@ class NewsWrapper:
     def clear_data(self):
         """Clear the stored raw data"""
         self.data = []
+        self.data_size = 0
         self.status_code = None
 
     def hard_reset(self, wait: bool = True, send_quit: bool = True):
@@ -324,9 +337,8 @@ class NNTP:
     def connect(self):
         """Start of connection, can be performed a-sync"""
         try:
-            # Wait only 15 seconds during server test
-            if self.nw.blocking:
-                self.sock.settimeout(15)
+            # Wait the defined timeout during connect and SSL-setup
+            self.sock.settimeout(self.nw.server.timeout)
 
             # Connect
             self.sock.connect((self.host, self.nw.server.port))
@@ -344,12 +356,11 @@ class NNTP:
                 )
                 self.nw.server.ssl_info = "%s (%s)" % (self.sock.version(), self.sock.cipher()[0])
 
-            # Set blocking mode
-            self.sock.setblocking(self.nw.blocking)
-
-            # Now it's safe to add the socket to the list of active sockets
-            # Skip this step during server test
+            # Skip during server test
             if not self.nw.blocking:
+                # Set to non-blocking mode
+                self.sock.setblocking(False)
+                # Now it's safe to add the socket to the list of active sockets
                 sabnzbd.Downloader.add_socket(self.fileno, self.nw)
         except OSError as e:
             self.error(e)
@@ -390,10 +401,13 @@ class NNTP:
             raise socket.error(errno.ECONNREFUSED, str(error))
         else:
             msg = "Failed to connect: %s" % (str(error))
-            msg = "%s %s@%s:%s" % (msg, self.nw.thrdnum, self.host, self.nw.server.port)
+            msg = "%s %s:%s (%s)" % (msg, self.nw.server.host, self.nw.server.port, self.host)
             self.error_msg = msg
             self.nw.server.next_busy_threads_check = 0
-            logging.info(msg)
+            if self.nw.server.warning == msg:
+                logging.info(msg)
+            else:
+                logging.warning(msg)
             self.nw.server.warning = msg
 
     def __repr__(self):
