@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2021 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -22,13 +22,12 @@ sabnzbd.nzbqueue - nzb queue
 import os
 import logging
 import time
-import datetime
-import functools
+import cherrypy._cpreqbody
 from typing import List, Dict, Union, Tuple, Optional
 
 import sabnzbd
 from sabnzbd.nzbstuff import NzbObject, Article
-from sabnzbd.misc import exit_sab, cat_to_opts, int_conv, caller_name, cmp, safe_lower
+from sabnzbd.misc import exit_sab, cat_to_opts, int_conv, caller_name, safe_lower
 from sabnzbd.filesystem import get_admin_path, remove_all, globber_full, remove_file, is_valid_script
 from sabnzbd.nzbparser import process_single_nzb
 from sabnzbd.panic import panic_queue
@@ -47,26 +46,24 @@ from sabnzbd.constants import (
     STOP_PRIORITY,
     VERIFIED_FILE,
     Status,
-    IGNORED_FOLDERS,
-    QNFO,
+    IGNORED_FILES_AND_FOLDERS,
     DIRECT_WRITE_TRIGGER,
 )
 
 import sabnzbd.cfg as cfg
 from sabnzbd.downloader import Server
-from sabnzbd.assembler import file_has_articles
 import sabnzbd.notifier as notifier
 
 
 class NzbQueue:
-    """ Singleton NzbQueue """
+    """Singleton NzbQueue"""
 
     def __init__(self):
         self.__top_only: bool = cfg.top_only()
         self.__nzo_list: List[NzbObject] = []
         self.__nzo_table: Dict[str, NzbObject] = {}
 
-    def read_queue(self, repair):
+    def read_queue(self, repair: int):
         """Read queue from disk, supporting repair modes
         0 = no repairs
         1 = use existing queue, add missing "incomplete" folders
@@ -76,7 +73,7 @@ class NzbQueue:
         if repair < 2:
             # Try to process the queue file
             try:
-                data = sabnzbd.load_admin(QUEUE_FILE_NAME)
+                data = sabnzbd.filesystem.load_admin(QUEUE_FILE_NAME)
                 if data:
                     queue_vers, nzo_ids, _ = data
                     if not queue_vers == QUEUE_VERSION:
@@ -99,11 +96,11 @@ class NzbQueue:
             path = get_admin_path(folder, future=False)
 
             # Try as normal job
-            nzo = sabnzbd.load_data(_id, path, remove=False)
+            nzo = sabnzbd.filesystem.load_data(_id, path, remove=False)
             if not nzo:
                 # Try as future job
                 path = get_admin_path(folder, future=True)
-                nzo = sabnzbd.load_data(_id, path)
+                nzo = sabnzbd.filesystem.load_data(_id, path)
             if nzo:
                 self.add(nzo, save=False, quiet=True)
                 folders.append(folder)
@@ -117,7 +114,7 @@ class NzbQueue:
                 path, nzo_id = os.path.split(item)
                 if nzo_id not in self.__nzo_table:
                     if nzo_id.startswith("SABnzbd_nzo"):
-                        nzo = sabnzbd.load_data(nzo_id, path, remove=True)
+                        nzo = sabnzbd.filesystem.load_data(nzo_id, path, remove=True)
                         if nzo:
                             self.add(nzo, save=True)
                     else:
@@ -127,7 +124,7 @@ class NzbQueue:
                             pass
 
     @NzbQueueLocker
-    def scan_jobs(self, all_jobs=False, action=True):
+    def scan_jobs(self, all_jobs: bool = False, action: bool = True) -> List[str]:
         """Scan "incomplete" for missing folders,
         'all' is True: Include active folders
         'action' is True, do the recovery action
@@ -154,7 +151,7 @@ class NzbQueue:
         # Repair unregistered folders
         for folder in globber_full(cfg.download_dir.get_path()):
             name = os.path.basename(folder)
-            if os.path.isdir(folder) and name not in registered and name not in IGNORED_FOLDERS:
+            if os.path.isdir(folder) and name not in registered and name not in IGNORED_FILES_AND_FOLDERS:
                 if action:
                     logging.info("Repairing job %s", folder)
                     self.repair_job(folder)
@@ -164,8 +161,10 @@ class NzbQueue:
                     logging.info("Skipping repair for job %s", folder)
         return result
 
-    def repair_job(self, repair_folder, new_nzb=None, password=None):
-        """ Reconstruct admin for a single job folder, optionally with new NZB """
+    def repair_job(
+        self, repair_folder: str, new_nzb: Optional[cherrypy._cpreqbody.Part] = None, password: Optional[str] = None
+    ) -> Optional[str]:
+        """Reconstruct admin for a single job folder, optionally with new NZB"""
         # Check if folder exists
         if not repair_folder or not os.path.exists(repair_folder):
             return None
@@ -177,17 +176,19 @@ class NzbQueue:
         if getattr(new_nzb, "filename", None):
             remove_all(admin_path, "*.gz", keep_folder=True)
             logging.debug("Repair job %s with new NZB (%s)", name, new_nzb.filename)
-            _, nzo_ids = sabnzbd.add_nzbfile(new_nzb, nzbname=name, reuse=repair_folder, password=password)
+            _, nzo_ids = sabnzbd.nzbparser.add_nzbfile(new_nzb, nzbname=name, reuse=repair_folder, password=password)
         else:
             # Was this file already post-processed?
-            verified = sabnzbd.load_data(VERIFIED_FILE, admin_path, remove=False)
+            verified = sabnzbd.filesystem.load_data(VERIFIED_FILE, admin_path, remove=False)
             filenames = []
             if not verified or not all(verified[x] for x in verified):
                 filenames = globber_full(admin_path, "*.gz")
 
             if filenames:
                 logging.debug("Repair job %s by re-parsing stored NZB", name)
-                _, nzo_ids = sabnzbd.add_nzbfile(filenames[0], nzbname=name, reuse=repair_folder, password=password)
+                _, nzo_ids = sabnzbd.nzbparser.add_nzbfile(
+                    filenames[0], nzbname=name, reuse=repair_folder, password=password
+                )
             else:
                 try:
                     logging.debug("Repair job %s without stored NZB", name)
@@ -207,7 +208,7 @@ class NzbQueue:
 
     @NzbQueueLocker
     def send_back(self, old_nzo: NzbObject):
-        """ Send back job to queue after successful pre-check """
+        """Send back job to queue after successful pre-check"""
         try:
             nzb_path = globber_full(old_nzo.admin_path, "*.gz")[0]
         except:
@@ -229,28 +230,30 @@ class NzbQueue:
 
     @NzbQueueLocker
     def save(self, save_nzo: Union[NzbObject, None, bool] = None):
-        """ Save queue, all nzo's or just the specified one """
+        """Save queue, all nzo's or just the specified one"""
         logging.info("Saving queue")
 
         nzo_ids = []
         # Aggregate nzo_ids and save each nzo
         for nzo in self.__nzo_list[:]:
-            if not nzo.is_gone():
+            if not nzo.removed_from_queue:
                 nzo_ids.append(os.path.join(nzo.work_name, nzo.nzo_id))
                 if save_nzo is None or nzo is save_nzo:
                     if not nzo.futuretype:
                         # Also includes save_data for NZO
                         nzo.save_to_disk()
                     else:
-                        sabnzbd.save_data(nzo, nzo.nzo_id, nzo.admin_path)
+                        sabnzbd.filesystem.save_data(nzo, nzo.nzo_id, nzo.admin_path)
 
-        sabnzbd.save_admin((QUEUE_VERSION, nzo_ids, []), QUEUE_FILE_NAME)
+        sabnzbd.filesystem.save_admin((QUEUE_VERSION, nzo_ids, []), QUEUE_FILE_NAME)
 
     def set_top_only(self, value):
         self.__top_only = value
 
-    def generate_future(self, msg, pp=None, script=None, cat=None, url=None, priority=DEFAULT_PRIORITY, nzbname=None):
-        """ Create and return a placeholder nzo object """
+    def generate_future(
+        self, msg, pp=None, script=None, cat=None, url=None, priority=DEFAULT_PRIORITY, nzbname=None
+    ) -> NzbObject:
+        """Create and return a placeholder nzo object"""
         logging.debug("Creating placeholder NZO")
         future_nzo = NzbObject(
             filename=msg,
@@ -284,7 +287,7 @@ class NzbQueue:
                     result += 1
         return result
 
-    def change_cat(self, nzo_ids: str, cat: str, explicit_priority=None):
+    def change_cat(self, nzo_ids: str, cat: str) -> int:
         result = 0
         for nzo_id in [item.strip() for item in nzo_ids.split(",")]:
             if nzo_id in self.__nzo_table:
@@ -292,14 +295,13 @@ class NzbQueue:
                 nzo.cat, pp, nzo.script, prio = cat_to_opts(cat)
                 logging.info("Set cat=%s for job %s", cat, nzo.final_name)
                 nzo.set_pp(pp)
-                if explicit_priority is None:
-                    self.set_priority(nzo_id, prio)
+                self.set_priority(nzo_id, prio)
                 # Abort any ongoing unpacking if the category changed
                 nzo.abort_direct_unpacker()
                 result += 1
         return result
 
-    def change_name(self, nzo_id: str, name: str, password: str = None):
+    def change_name(self, nzo_id: str, name: str, password: str = None) -> bool:
         if nzo_id in self.__nzo_table:
             nzo = self.__nzo_table[nzo_id]
             logging.info("Renaming %s to %s", nzo.final_name, name)
@@ -322,9 +324,10 @@ class NzbQueue:
             return None
 
     @NzbQueueLocker
-    def add(self, nzo: NzbObject, save=True, quiet=False) -> str:
+    def add(self, nzo: NzbObject, save: bool = True, quiet: bool = False) -> str:
+        # Can already be set for future jobs
         if not nzo.nzo_id:
-            nzo.nzo_id = sabnzbd.get_new_id("nzo", nzo.admin_path, self.__nzo_table)
+            nzo.nzo_id = sabnzbd.filesystem.get_new_id("nzo", nzo.admin_path, self.__nzo_table)
 
         # If no files are to be downloaded anymore, send to postproc
         if not nzo.files and not nzo.futuretype:
@@ -333,7 +336,7 @@ class NzbQueue:
 
         # Reset try_lists, markers and evaluate the scheduling settings
         nzo.reset_try_list()
-        nzo.deleted = False
+        nzo.removed_from_queue = False
         priority = nzo.priority
         if sabnzbd.Scheduler.analyse(False, priority):
             nzo.status = Status.PAUSED
@@ -380,7 +383,7 @@ class NzbQueue:
         return nzo.nzo_id
 
     @NzbQueueLocker
-    def remove(self, nzo_id: str, cleanup=True, delete_all_data=True):
+    def remove(self, nzo_id: str, cleanup: bool = True, delete_all_data: bool = True) -> Optional[str]:
         """Remove NZO from queue.
         It can be added to history directly.
         Or, we do some clean-up, sometimes leaving some data.
@@ -390,12 +393,10 @@ class NzbQueue:
             logging.info("[%s] Removing job %s", caller_name(), nzo.final_name)
 
             # Set statuses
-            nzo.deleted = True
-            if cleanup and not nzo.is_gone():
-                nzo.status = Status.DELETED
+            nzo.removed_from_queue = True
             self.__nzo_list.remove(nzo)
-
             if cleanup:
+                nzo.status = Status.DELETED
                 nzo.purge_data(delete_all_data=delete_all_data)
             self.save(False)
             return nzo_id
@@ -416,8 +417,8 @@ class NzbQueue:
         return removed
 
     @NzbQueueLocker
-    def remove_all(self, search: str = "") -> List[str]:
-        """ Remove NZO's that match the search-pattern """
+    def remove_all(self, search: Optional[str] = None) -> List[str]:
+        """Remove NZO's that match the search-pattern"""
         nzo_ids = []
         search = safe_lower(search)
         for nzo_id, nzo in self.__nzo_table.items():
@@ -425,29 +426,30 @@ class NzbQueue:
                 nzo_ids.append(nzo_id)
         return self.remove_multiple(nzo_ids)
 
-    def remove_nzf(self, nzo_id: str, nzf_id: str, force_delete=False) -> List[str]:
+    def remove_nzfs(self, nzo_id: str, nzf_ids: List[str]) -> List[str]:
         removed = []
         if nzo_id in self.__nzo_table:
             nzo = self.__nzo_table[nzo_id]
-            nzf = nzo.get_nzf_by_id(nzf_id)
 
-            if nzf:
-                removed.append(nzf_id)
-                nzo.abort_direct_unpacker()
-                post_done = nzo.remove_nzf(nzf)
-                if post_done:
-                    if nzo.finished_files:
-                        self.end_job(nzo)
+            for nzf_id in nzf_ids:
+                nzf = nzo.get_nzf_by_id(nzf_id)
+                if nzf:
+                    removed.append(nzf_id)
+                    nzo.abort_direct_unpacker()
+                    post_done = nzo.remove_nzf(nzf)
+                    if post_done:
+                        if nzo.finished_files:
+                            self.end_job(nzo)
+                        else:
+                            self.remove(nzo_id)
                     else:
-                        self.remove(nzo_id)
-                elif force_delete:
-                    # Force-remove all trace and update counters
-                    nzo.bytes -= nzf.bytes
-                    nzo.bytes_tried -= nzf.bytes - nzf.bytes_left
-                    if nzf.is_par2 or sabnzbd.par2file.is_parfile(nzf.filename):
-                        nzo.bytes_par2 -= nzf.bytes
-                    del nzo.files_table[nzf_id]
-                    nzo.finished_files.remove(nzf)
+                        # Force-remove all trace and update counters
+                        nzo.bytes -= nzf.bytes
+                        nzo.bytes_tried -= nzf.bytes - nzf.bytes_left
+                        if nzf.is_par2 or sabnzbd.par2file.is_parfile(nzf.filename):
+                            nzo.bytes_par2 -= nzf.bytes
+                        del nzo.files_table[nzf_id]
+                        nzo.finished_files.remove(nzf)
             logging.info("Removed NZFs %s from job %s", removed, nzo.final_name)
         return removed
 
@@ -540,65 +542,67 @@ class NzbQueue:
         return -1, nzo1.priority
 
     @NzbQueueLocker
-    def move_up_bulk(self, nzo_id, nzf_ids, size):
+    def move_nzf_up_bulk(self, nzo_id: str, nzf_ids: List[str], size: int):
         if nzo_id in self.__nzo_table:
-            for unused in range(size):
+            for _ in range(size):
                 self.__nzo_table[nzo_id].move_up_bulk(nzf_ids)
 
     @NzbQueueLocker
-    def move_top_bulk(self, nzo_id, nzf_ids):
+    def move_nzf_top_bulk(self, nzo_id: str, nzf_ids: List[str]):
         if nzo_id in self.__nzo_table:
             self.__nzo_table[nzo_id].move_top_bulk(nzf_ids)
 
     @NzbQueueLocker
-    def move_down_bulk(self, nzo_id, nzf_ids, size):
+    def move_nzf_down_bulk(self, nzo_id: str, nzf_ids: List[str], size: int):
         if nzo_id in self.__nzo_table:
-            for unused in range(size):
+            for _ in range(size):
                 self.__nzo_table[nzo_id].move_down_bulk(nzf_ids)
 
     @NzbQueueLocker
-    def move_bottom_bulk(self, nzo_id, nzf_ids):
+    def move_nzf_bottom_bulk(self, nzo_id: str, nzf_ids: List[str]):
         if nzo_id in self.__nzo_table:
             self.__nzo_table[nzo_id].move_bottom_bulk(nzf_ids)
 
     @NzbQueueLocker
-    def sort_by_avg_age(self, reverse=False):
-        logging.info("Sorting by average date... (reversed: %s)", reverse)
-        self.__nzo_list = sort_queue_function(self.__nzo_list, _nzo_date_cmp, reverse)
-
-    @NzbQueueLocker
-    def sort_by_name(self, reverse=False):
-        logging.info("Sorting by name... (reversed: %s)", reverse)
-        self.__nzo_list = sort_queue_function(self.__nzo_list, _nzo_name_cmp, reverse)
-
-    @NzbQueueLocker
-    def sort_by_size(self, reverse=False):
-        logging.info("Sorting by size... (reversed: %s)", reverse)
-        self.__nzo_list = sort_queue_function(self.__nzo_list, _nzo_size_cmp, reverse)
-
-    def sort_queue(self, field, reverse=None):
-        """Sort queue by field: "name", "size" or "avg_age"
-        Direction is specified as "desc"/True or "asc"/False
+    def sort_queue(self, field: str, direction: Optional[str] = None):
+        """Sort queue by field: "name", "size" or "avg_age" or by percentage remaining
+        Direction is specified as "desc" or "asc"
         """
-        if isinstance(reverse, str):
-            if reverse.lower() == "desc":
-                reverse = True
-            else:
-                reverse = False
-        if reverse is None:
-            reverse = False
-        if field.lower() == "name":
-            self.sort_by_name(reverse)
-        elif field.lower() == "size" or field.lower() == "bytes":
-            self.sort_by_size(reverse)
-        elif field.lower() == "avg_age":
-            self.sort_by_avg_age(not reverse)
+        field = field.lower()
+        reverse = False
+        if safe_lower(direction) == "desc":
+            reverse = True
+
+        if field == "name":
+            logging.info("Sorting by name (reversed: %s)", reverse)
+            sort_function = lambda nzo: nzo.final_name.lower()
+        elif field == "size" or field == "bytes":
+            logging.info("Sorting by size (reversed: %s)", reverse)
+            sort_function = lambda nzo: nzo.bytes
+        elif field == "avg_age":
+            reverse = not reverse
+            logging.info("Sorting by average date... (reversed: %s)", reverse)
+            sort_function = lambda nzo: nzo.avg_date
+        elif field == "remaining":
+            logging.debug("Sorting by percentage downloaded...")
+            sort_function = lambda nzo: nzo.remaining / nzo.bytes
         else:
             logging.debug("Sort: %s not recognized", field)
+            return
+
+        # Apply sort by requested order, then restore priority ordering
+        self.__nzo_list.sort(key=sort_function, reverse=reverse)
+        self.__nzo_list.sort(key=lambda nzo: nzo.priority, reverse=True)
+
+    def update_sort_order(self):
+        """Resorts the queue if it is useful for the selected sort method"""
+        auto_sort = cfg.auto_sort()
+        if auto_sort and auto_sort.startswith("remaining"):
+            self.sort_queue("remaining")
 
     @NzbQueueLocker
-    def __set_priority(self, nzo_id, priority):
-        """ Sets the priority on the nzo and places it in the queue at the appropriate position """
+    def __set_priority(self, nzo_id: str, priority: Union[int, str]) -> Optional[int]:
+        """Sets the priority on the nzo and places it in the queue at the appropriate position"""
         try:
             priority = int_conv(priority)
             nzo = self.__nzo_table[nzo_id]
@@ -675,7 +679,7 @@ class NzbQueue:
             return -1
 
     @NzbQueueLocker
-    def set_priority(self, nzo_ids, priority):
+    def set_priority(self, nzo_ids: str, priority: int) -> int:
         try:
             n = -1
             for nzo_id in [item.strip() for item in nzo_ids.split(",")]:
@@ -685,24 +689,29 @@ class NzbQueue:
             return -1
 
     @staticmethod
-    def reset_try_lists(article: Article, article_reset=True):
-        """ Let article get new fetcher and reset trylists """
+    def reset_try_lists(article: Article, remove_fetcher_from_trylist: bool = True):
+        """Let article get new fetcher and reset trylists"""
+        if remove_fetcher_from_trylist:
+            article.remove_from_try_list(article.fetcher)
         article.fetcher = None
-        if article_reset:
-            article.reset_try_list()
+        article.tries = 0
         article.nzf.reset_try_list()
         article.nzf.nzo.reset_try_list()
 
-    def has_forced_items(self):
+    def has_forced_jobs(self) -> bool:
         """Check if the queue contains any Forced
-        Priority items to download while paused
+        Priority jobs to download while paused
         """
         for nzo in self.__nzo_list:
             if nzo.priority == FORCE_PRIORITY and nzo.status not in (Status.PAUSED, Status.GRABBING):
                 return True
+            # Each time the priority of a job is changed the queue is sorted, so we can
+            # assume that if we reach a job below Force priority we can continue
+            if nzo.priority < FORCE_PRIORITY:
+                return False
         return False
 
-    def get_article(self, server: Server, servers: List[Server]) -> Optional[Article]:
+    def get_articles(self, server: Server, servers: List[Server], fetch_limit: int) -> List[Article]:
         """Get next article for jobs in the queue
         Not locked for performance, since it only reads the queue
         """
@@ -718,12 +727,13 @@ class NzbQueue:
                     or (nzo.avg_stamp + propagation_delay) < time.time()
                 ):
                     if not nzo.server_in_try_list(server):
-                        article = nzo.get_article(server, servers)
-                        if article:
-                            return article
+                        articles = nzo.get_articles(server, servers, fetch_limit)
+                        if articles:
+                            return articles
                     # Stop after first job that wasn't paused/propagating/etc
                     if self.__top_only:
-                        return
+                        return []
+        return []
 
     def register_article(self, article: Article, success: bool = True):
         """Register the articles we tried
@@ -732,49 +742,54 @@ class NzbQueue:
         nzf = article.nzf
         nzo = nzf.nzo
 
-        if nzf.deleted:
-            logging.debug("Discarding article %s, no longer in queue", article.article)
+        if nzo.pp_or_finished or nzf.deleted:
+            logging.debug("Discarding article for file %s: deleted or already post-processing", nzf.filename)
+            # If this file is needed later (par2 file added back to queue), it would be damaged because
+            # we discard this article. So we reset it to be picked up again if needed.
+            # But not reset all articles, as it could cause problems for articles still attached to a server.
+            article.reset_try_list()
+            nzf.reset_try_list()
             return
 
         articles_left, file_done, post_done = nzo.remove_article(article, success)
 
-        if nzo.is_gone():
-            logging.debug("Discarding article for file %s, no longer in queue", nzf.filename)
-        else:
-            # Write data if file is done or at trigger time
-            if file_done or (articles_left and (articles_left % DIRECT_WRITE_TRIGGER) == 0):
-                if not nzo.precheck:
-                    # Only start decoding if we have a filename and type
-                    # The type is only set if sabyenc could decode the article
-                    if nzf.filename and nzf.type:
-                        sabnzbd.Assembler.process(nzo, nzf, file_done)
-                    elif nzf.filename.lower().endswith(".par2"):
-                        # Broken par2 file, try to get another one
-                        nzo.promote_par2(nzf)
-                    else:
-                        if file_has_articles(nzf):
-                            logging.warning(T("%s -> Unknown encoding"), nzf.filename)
+        # Write data if file is done or at trigger time
+        # Skip if the file is already queued, since all available articles will then be written
+        if file_done or (
+            articles_left
+            and (articles_left % DIRECT_WRITE_TRIGGER) == 0
+            and not sabnzbd.Assembler.partial_nzf_in_queue(nzf)
+        ):
+            if not nzo.precheck:
+                # Only start decoding if we have a filename and type
+                # The type is only set if sabyenc could decode the article
+                if nzf.filename and nzf.type:
+                    sabnzbd.Assembler.process(nzo, nzf, file_done)
+                elif nzf.filename.lower().endswith(".par2"):
+                    # Broken par2 file, try to get another one
+                    nzo.promote_par2(nzf)
 
-            # Save bookkeeping in case of crash
-            if file_done and (nzo.next_save is None or time.time() > nzo.next_save):
-                nzo.save_to_disk()
-                sabnzbd.BPSMeter.save()
-                if nzo.save_timeout is None:
-                    nzo.next_save = None
-                else:
-                    nzo.next_save = time.time() + nzo.save_timeout
+        # Save bookkeeping in case of crash
+        if file_done and (nzo.next_save is None or time.time() > nzo.next_save):
+            nzo.save_to_disk()
+            sabnzbd.BPSMeter.save()
+            if nzo.save_timeout is None:
+                nzo.next_save = None
+            else:
+                nzo.next_save = time.time() + nzo.save_timeout
 
-            # Remove post from Queue
-            if post_done:
-                nzo.set_download_report()
-                self.end_job(nzo)
+        # Remove post from Queue
+        if post_done:
+            nzo.set_download_report()
+            self.end_job(nzo)
 
+    @NzbQueueLocker
     def end_job(self, nzo: NzbObject):
-        """ Send NZO to the post-processing queue """
+        """Send NZO to the post-processing queue"""
         # Notify assembler to call postprocessor
-        if not nzo.deleted:
+        if not nzo.removed_from_queue:
             logging.info("[%s] Ending job %s", caller_name(), nzo.final_name)
-            nzo.deleted = True
+            nzo.removed_from_queue = True
             if nzo.precheck:
                 nzo.save_to_disk()
                 # Check result
@@ -788,7 +803,7 @@ class NzbQueue:
                     pass
             sabnzbd.Assembler.process(nzo)
 
-    def actives(self, grabs=True) -> int:
+    def actives(self, grabs: bool = True) -> int:
         """Return amount of non-paused jobs, optionally with 'grabbing' items
         Not locked for performance, only reads the queue
         """
@@ -801,21 +816,27 @@ class NzbQueue:
                 n += 1
         return n
 
-    def queue_info(self, search=None, nzo_ids=None, start=0, limit=0):
+    def queue_info(
+        self,
+        search: Optional[str] = None,
+        categories: Optional[List[str]] = None,
+        priorities: Optional[List[str]] = None,
+        nzo_ids: Optional[List[str]] = None,
+        start: int = 0,
+        limit: int = 0,
+    ) -> Tuple[int, int, int, List[NzbObject], int, int]:
         """Return list of queued jobs,
         optionally filtered by 'search' and 'nzo_ids', and limited by start and limit.
         Not locked for performance, only reads the queue
         """
         if search:
             search = search.lower()
-        if nzo_ids:
-            nzo_ids = nzo_ids.split(",")
         bytes_left = 0
         bytes_total = 0
         bytes_left_previous_page = 0
         q_size = 0
-        pnfo_list = []
-        n = 0
+        nzo_list = []
+        nzos_matched = 0
 
         for nzo in self.__nzo_list:
             if nzo.status not in (Status.PAUSED, Status.CHECKING) or nzo.priority == FORCE_PRIORITY:
@@ -824,20 +845,28 @@ class NzbQueue:
                 bytes_left += b_left
                 q_size += 1
                 # We need the number of bytes before the current page
-                if n < start:
+                if nzos_matched < start:
                     bytes_left_previous_page += b_left
 
-            if (not search) or search in nzo.final_name.lower():
-                if (not nzo_ids) or nzo.nzo_id in nzo_ids:
-                    if (not limit) or (start <= n < start + limit):
-                        pnfo_list.append(nzo.gather_info())
-                    n += 1
+            # Conditions split up for readability
+            if search and search not in nzo.final_name.lower():
+                continue
+            if categories and nzo.cat not in categories:
+                continue
+            if priorities and nzo.priority not in priorities:
+                continue
+            if nzo_ids and nzo.nzo_id not in nzo_ids:
+                continue
+
+            if not limit or start <= nzos_matched < start + limit:
+                nzo_list.append(nzo)
+            nzos_matched += 1
 
         if not search and not nzo_ids:
-            n = len(self.__nzo_list)
-        return QNFO(bytes_total, bytes_left, bytes_left_previous_page, pnfo_list, q_size, n)
+            nzos_matched = len(self.__nzo_list)
+        return bytes_total, bytes_left, bytes_left_previous_page, nzo_list, q_size, nzos_matched
 
-    def remaining(self):
+    def remaining(self) -> int:
         """Return bytes left in the queue by non-paused items
         Not locked for performance, only reads the queue
         """
@@ -847,7 +876,7 @@ class NzbQueue:
                 bytes_left += nzo.remaining
         return bytes_left
 
-    def is_empty(self):
+    def is_empty(self) -> bool:
         empty = True
         for nzo in self.__nzo_list:
             if not nzo.futuretype and nzo.status != Status.PAUSED:
@@ -856,8 +885,16 @@ class NzbQueue:
         return empty
 
     def stop_idle_jobs(self):
-        """ Detect jobs that have zero files left and send them to post processing """
+        """Detect jobs that have zero files left and send them to post processing"""
+        # Only check servers that are active
+        active_servers = [server for server in sabnzbd.Downloader.servers[:] if server.active]
+        nr_servers = len(active_servers)
         empty = []
+
+        if nr_servers <= 0:
+            logging.debug("Skipping stop_idle_jobs because no servers are active")
+            return
+
         for nzo in self.__nzo_list:
             if not nzo.futuretype and not nzo.files and nzo.status not in (Status.PAUSED, Status.GRABBING):
                 logging.info("Found idle job %s", nzo.final_name)
@@ -865,11 +902,19 @@ class NzbQueue:
 
             # Stall prevention by checking if all servers are in the trylist
             # This is a CPU-cheaper alternative to prevent stalling
-            if len(nzo.try_list) == sabnzbd.Downloader.server_nr:
+            if len(nzo.try_list) >= nr_servers:
                 # Maybe the NZF's need a reset too?
                 for nzf in nzo.files:
-                    if len(nzf.try_list) == sabnzbd.Downloader.server_nr:
-                        # We do not want to reset all article trylists, they are good
+                    if nzo.removed_from_queue:
+                        break
+
+                    if len(nzf.try_list) >= nr_servers:
+                        # Check for articles where all active servers have already been tried
+                        for article in nzf.articles[:]:
+                            if article.all_servers_in_try_list(active_servers):
+                                sabnzbd.NzbQueue.register_article(article, success=False)
+                                nzo.increase_bad_articles_counter("missing_articles")
+
                         logging.info("Resetting bad trylist for file %s in job %s", nzf.filename, nzo.final_name)
                         nzf.reset_try_list()
 
@@ -904,8 +949,8 @@ class NzbQueue:
                 # Don't use nzo.resume() to avoid resetting job warning flags
                 nzo.status = Status.QUEUED
 
-    def get_urls(self):
-        """ Return list of future-types needing URL """
+    def get_urls(self) -> List[Tuple[str, NzbObject]]:
+        """Return list of future-types needing URL"""
         lst = []
         for nzo_id in self.__nzo_table:
             nzo = self.__nzo_table[nzo_id]
@@ -917,53 +962,3 @@ class NzbQueue:
 
     def __repr__(self):
         return "<NzbQueue>"
-
-
-def _nzo_date_cmp(nzo1: NzbObject, nzo2: NzbObject):
-    avg_date1 = nzo1.avg_date
-    avg_date2 = nzo2.avg_date
-
-    if avg_date1 is None and avg_date2 is None:
-        return 0
-
-    if avg_date1 is None:
-        avg_date1 = datetime.datetime.now()
-    elif avg_date2 is None:
-        avg_date2 = datetime.datetime.now()
-
-    return cmp(avg_date1, avg_date2)
-
-
-def _nzo_name_cmp(nzo1, nzo2):
-    return cmp(nzo1.final_name.lower(), nzo2.final_name.lower())
-
-
-def _nzo_size_cmp(nzo1, nzo2):
-    return cmp(nzo1.bytes, nzo2.bytes)
-
-
-def sort_queue_function(nzo_list: List[NzbObject], method, reverse: bool) -> List[NzbObject]:
-    ultra_high_priority = [nzo for nzo in nzo_list if nzo.priority == REPAIR_PRIORITY]
-    super_high_priority = [nzo for nzo in nzo_list if nzo.priority == FORCE_PRIORITY]
-    high_priority = [nzo for nzo in nzo_list if nzo.priority == HIGH_PRIORITY]
-    normal_priority = [nzo for nzo in nzo_list if nzo.priority == NORMAL_PRIORITY]
-    low_priority = [nzo for nzo in nzo_list if nzo.priority == LOW_PRIORITY]
-
-    ultra_high_priority.sort(key=functools.cmp_to_key(method), reverse=reverse)
-    super_high_priority.sort(key=functools.cmp_to_key(method), reverse=reverse)
-    high_priority.sort(key=functools.cmp_to_key(method), reverse=reverse)
-    normal_priority.sort(key=functools.cmp_to_key(method), reverse=reverse)
-    low_priority.sort(key=functools.cmp_to_key(method), reverse=reverse)
-
-    new_list = ultra_high_priority
-    new_list.extend(super_high_priority)
-    new_list.extend(high_priority)
-    new_list.extend(normal_priority)
-    new_list.extend(low_priority)
-
-    # Make sure any left-over jobs enter the new list
-    for item in nzo_list:
-        if item not in new_list:
-            new_list.append(item)
-
-    return new_list
