@@ -23,6 +23,7 @@ import logging
 import os
 import re
 import struct
+from dataclasses import dataclass
 from typing import Dict, Optional, Tuple, BinaryIO
 
 from sabnzbd.constants import MEBI
@@ -37,6 +38,16 @@ PAR_CREATOR_ID = b"PAR 2.0\x00Creator\x00"
 PAR_RECOVERY_ID = b"RecvSlic"
 
 
+@dataclass
+class FilePar2Info:
+    """Class for keeping track of par2 information of a file"""
+
+    hash16k: bytes
+    filehash: bytes
+    filesize: int
+    has_duplicate: bool = False
+
+
 def is_parfile(filename: str) -> bool:
     """Check quickly whether file has par2 signature
     or if the filename has '.par2' in it
@@ -86,7 +97,7 @@ def analyse_par2(name: str, filepath: Optional[str] = None) -> Tuple[str, int, i
     return setname, vol, block
 
 
-def parse_par2_file(fname: str, md5of16k: Dict[bytes, str]) -> Dict[str, bytes]:
+def parse_par2_file(fname: str, md5of16k: Dict[bytes, str]) -> Tuple[str, Dict[str, FilePar2Info]]:
     """Get the hash table and the first-16k hash table from a PAR2 file
     Return as dictionary, indexed on names or hashes for the first-16 table
     The input md5of16k is modified in place and thus not returned!
@@ -104,15 +115,16 @@ def parse_par2_file(fname: str, md5of16k: Dict[bytes, str]) -> Dict[str, bytes]:
             header = f.read(8)
             while header:
                 if header == PAR_PKT_ID:
-                    name, filehash, hash16k, nr_files = parse_par2_packet(f)
+                    name, filehash, hash16k, filesize, set_id, nr_files = parse_par2_packet(f)
                     if name:
-                        table[name] = filehash
+                        table[name] = FilePar2Info(hash16k, filehash, filesize)
                         if hash16k not in md5of16k:
                             md5of16k[hash16k] = name
                         elif md5of16k[hash16k] != name:
                             # Not unique and not already linked to this file
-                            # Remove to avoid false-renames
+                            # Mark and remove to avoid false-renames
                             duplicates16k.append(hash16k)
+                            table[name].has_duplicate = True
 
                     # Store the number of files for later
                     if nr_files:
@@ -136,13 +148,15 @@ def parse_par2_file(fname: str, md5of16k: Dict[bytes, str]) -> Dict[str, bytes]:
             old_name = md5of16k.pop(hash16k)
             logging.debug("Par2-16k signature of %s not unique, discarding", old_name)
 
-    return table
+    return set_id, table
 
 
-def parse_par2_packet(f: BinaryIO) -> Tuple[Optional[str], Optional[bytes], Optional[bytes], Optional[int]]:
+def parse_par2_packet(
+    f: BinaryIO,
+) -> Tuple[Optional[str], Optional[bytes], Optional[bytes], Optional[int], Optional[str], Optional[int]]:
     """Look up and analyze a PAR2 packet"""
 
-    filename, filehash, hash16k, nr_files = nothing = None, None, None, None
+    filename, filehash, hash16k, filesize, set_id, nr_files = nothing = None, None, None, None, None, None
 
     # All packages start with a header before the body
     # 8	  : PAR2\x00PKT
@@ -168,6 +182,9 @@ def parse_par2_packet(f: BinaryIO) -> Tuple[Optional[str], Optional[bytes], Opti
     if md5sum != md5.digest():
         return nothing
 
+    # Get the Recovery Set ID
+    set_id = data[:16].hex()
+
     # See if it's any of the packages we care about
     par2_packet_type = data[16:32]
 
@@ -181,6 +198,7 @@ def parse_par2_packet(f: BinaryIO) -> Tuple[Optional[str], Optional[bytes], Opti
         # xx : Name (multiple of 4, padded with \0 if needed)
         filehash = data[48:64]
         hash16k = data[64:80]
+        filesize = int.from_bytes(data[80:88], byteorder="little", signed=False)
         filename = correct_unknown_encoding(data[88:].strip(b"\0"))
     elif par2_packet_type == PAR_CREATOR_ID:
         # From here until the end is the creator-text
@@ -194,4 +212,4 @@ def parse_par2_packet(f: BinaryIO) -> Tuple[Optional[str], Optional[bytes], Opti
         # 4  : Number of files in the recovery set
         nr_files = struct.unpack("<I", data[40:44])[0]
 
-    return filename, filehash, hash16k, nr_files
+    return filename, filehash, hash16k, filesize, set_id, nr_files
