@@ -137,6 +137,8 @@ class Scheduler:
             elif action_name == "rss_scan":
                 action = sabnzbd.RSSReader.run
                 rss_planned = True
+            elif action_name == "create_backup":
+                action = sabnzbd.config.create_config_backup
             elif action_name == "remove_failed":
                 action = sabnzbd.api.history_remove_failed
             elif action_name == "remove_completed":
@@ -218,7 +220,7 @@ class Scheduler:
             sabnzbd.downloader.check_server_expiration, "check_server_expiration", DAILY_RANGE, None, (0, 0)
         )
 
-        logging.info("Setting scheduler for server quota check")
+        logging.info("Setting schedule for server quota check")
         self.scheduler.add_interval_task(
             sabnzbd.downloader.check_server_quota,
             "check_server_quota",
