@@ -42,29 +42,11 @@ import sabnzbd.notifier as notifier
 from sabnzbd.encoding import ubtou, utob
 from sabnzbd.nzbstuff import NzbObject
 
-_RARTING_FIELDS = (
-    "x-rating-id",
-    "x-rating-url",
-    "x-rating-host",
-    "x-rating-video",
-    "x-rating-videocnt",
-    "x-rating-audio",
-    "x-rating-audiocnt",
-    "x-rating-voteup",
-    "x-rating-votedown",
-    "x-rating-spam",
-    "x-rating-confirmed-spam",
-    "x-rating-passworded",
-    "x-rating-confirmed-passworded",
-)
-
 
 class URLGrabber(Thread):
     def __init__(self):
         super().__init__()
         self.queue: queue.Queue[Tuple[Optional[str], Optional[NzbObject]]] = queue.Queue()
-        for url_nzo_tup in sabnzbd.NzbQueue.get_urls():
-            self.queue.put(url_nzo_tup)
         self.shutdown = False
 
     def add(self, url: str, future_nzo: NzbObject, when: Optional[int] = None):
@@ -87,7 +69,11 @@ class URLGrabber(Thread):
         self.queue.put((None, None))
 
     def run(self):
-        self.shutdown = False
+        # Read all URL's to grab from the queue
+        for url_nzo_tup in sabnzbd.NzbQueue.get_urls():
+            self.queue.put(url_nzo_tup)
+
+        # Start fetching
         while not self.shutdown:
             # Set NzbObject object to None so reference from this thread
             # does not keep the object alive in the future (see #1628)
@@ -185,10 +171,6 @@ class URLGrabber(Thread):
                         elif item == "retry-after":
                             wait = misc.int_conv(value)
 
-                        # Rating fields
-                        if item in _RARTING_FIELDS:
-                            nzo_info[item] = value
-
                         # Get filename from Content-Disposition header
                         if not filename and "filename" in value:
                             filename = filename_from_content_disposition(value)
@@ -302,10 +284,13 @@ class URLGrabber(Thread):
         msg: message to be logged
         content: report in history that cause is a bad NZB file
         """
-        # Remove the "Trying to fetch" part
-        if url:
-            nzo.filename = url
-            nzo.final_name = url.strip()
+        # Overwrite the "Trying to fetch" temporary name
+        url = url.strip()
+        nzo.filename = url
+        nzo.final_name = url
+        if nzo.custom_name:
+            # Try to set a nice name, if available
+            nzo.final_name = "%s - %s" % (nzo.custom_name, url)
 
         if content:
             # Bad content
@@ -314,7 +299,8 @@ class URLGrabber(Thread):
             # Failed fetch
             msg = T("URL Fetching failed; %s") % msg
 
-        # Mark as failed
+        # Mark as failed and set the info why
+        nzo.set_unpack_info("Source", url)
         nzo.set_unpack_info("Source", msg)
         nzo.fail_msg = msg
 
@@ -403,7 +389,7 @@ def add_url(
     password: Optional[str] = None,
 ):
     """Add NZB based on a URL, attributes optional"""
-    if "http" not in url:
+    if not url.lower().startswith("http"):
         return
     if not pp or pp == "-1":
         pp = None
