@@ -5,20 +5,23 @@
 import time
 import os
 import sys
+import logging
+import time
 
 _DUMP_DATA_SIZE = 10 * 1024 * 1024
 _DUMP_DATA = os.urandom(_DUMP_DATA_SIZE)
 
 
-def diskspeedmeasure(my_dirname: str) -> float:
+def diskspeedmeasure(dirname: str) -> float:
     """Returns writing speed to my_dirname in MB/s
     method: keep writing a file, until certain time is passed.
     Then divide bytes written by time passed
     In case of problems (ie non-writable dir or file), return 0.0
     """
+    start = time.time()
     maxtime = 0.5  # sec
     total_written = 0
-    filename = os.path.join(my_dirname, "outputTESTING.txt")
+    filename = os.path.join(dirname, "outputTESTING.txt")
 
     try:
         # Use low-level I/O
@@ -42,9 +45,12 @@ def diskspeedmeasure(my_dirname: str) -> float:
         os.remove(filename)
     except (PermissionError, NotADirectoryError, FileNotFoundError):
         # Could not write, so ... report 0.0
+        logging.debug("Failed to measure disk speed on %s", dirname)
         return 0.0
 
-    return total_written / total_time / 1024 / 1024
+    megabyte_per_second = total_written / total_time / 1024 / 1024
+    logging.debug("Disk speed of %s = %.2f MB/s (in %.2f seconds)", dirname, megabyte_per_second, time.time() - start)
+    return megabyte_per_second
 
 
 if __name__ == "__main__":
