@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/python3
 # based on SysTrayIcon.py by Simon Brunning - simon@brunningonline.net
 # http://www.brunningonline.net/simon/blog/archives/001835.html
 # http://www.brunningonline.net/simon/blog/archives/SysTrayIcon.py.html
@@ -21,12 +21,11 @@ from time import sleep
 class SysTrayIconThread(Thread):
     QUIT = "QUIT"
     SPECIAL_ACTIONS = [QUIT]
-
     FIRST_ID = 1023
-    terminate = False
 
     def __init__(self, icon, hover_text, menu_options, on_quit=None, default_menu_index=None, window_class_name=None):
-        Thread.__init__(self)
+        super().__init__()
+        self.terminate = False
         self.icon = icon
         self.icons = {}
         self.hover_text = hover_text
@@ -80,12 +79,16 @@ class SysTrayIconThread(Thread):
         self.notify_id = None
         self.refresh_icon()
 
+    def stop(self):
+        self.terminate = True
+        sleep(0.15)
+
     def run(self):
         self.initialize()
         while not self.terminate:
             win32gui.PumpWaitingMessages()
             self.doUpdates()
-            sleep(0.100)
+            sleep(0.1)
         win32gui.Shell_NotifyIcon(win32gui.NIM_DELETE, (self.hwnd, 0))
 
     # Override this
@@ -177,7 +180,7 @@ class SysTrayIconThread(Thread):
             self.show_menu()
         elif lparam == win32con.WM_LBUTTONDOWN:
             # Wrapper of win32api, timeout is in ms
-            # We need to wait at least untill what user has defined as double click
+            # We need to wait at least until what user has defined as double click
             self.stop_click_timer()
             self.click_timer = timer.set_timer(win32gui.GetDoubleClickTime() * 2, self.click)
         return True
