@@ -89,13 +89,13 @@ class TestDeobfuscateFinalResult:
         deobfuscate(nzo, filelist, jobname)
 
     def test_deobfuscate_filelist_lite(self):
-        # ligthweight test of deobfuscating: with just one file
+        # lightweight test of deobfuscating: with just one file
 
         # Create directory (with a random directory name)
         dirname = os.path.join(SAB_CACHE_DIR, "testdir" + str(random.randint(10000, 99999)))
         os.mkdir(dirname)
 
-        # Create a big enough file with a non-useful, obfuscated filename
+        # Create a big file with a useless, obfuscated filename
         output_file1 = os.path.join(dirname, "111c1c9e2bdfb5114044bf25152b7eab.bin")
         create_big_file(output_file1)
         assert os.path.isfile(output_file1)
@@ -115,6 +115,56 @@ class TestDeobfuscateFinalResult:
         # Done. Remove (non-empty) directory
         shutil.rmtree(dirname)
 
+    def test_deobfuscate_big_file_small_accompanying_files(self):
+        # input: myiso.iso, with accompanying files (.srt and -sample files)
+        # test that the accompanying files (with same basename) are renamed accordingly to the big ISO
+        # Note: this is the most typical usage of deobfuscation
+
+        # Create directory (with a random directory name)
+        dirname = os.path.join(SAB_CACHE_DIR, "testdir" + str(random.randint(10000, 99999)))
+        os.mkdir(dirname)
+
+        # Create a big enough file with a useless filename
+        isofile = os.path.join(dirname, "myiso.iso")
+        create_big_file(isofile)
+        assert os.path.isfile(isofile)
+
+        # and a srt file
+        srtfile = os.path.join(dirname, "myiso.srt")
+        create_small_file(srtfile)
+        assert os.path.isfile(srtfile)
+
+        # and a sample file
+        samplefile = os.path.join(dirname, "myiso-sample.iso")
+        create_small_file(samplefile)
+        assert os.path.isfile(samplefile)
+
+        # and a non-related file
+        txtfile = os.path.join(dirname, "something.txt")
+        create_small_file(txtfile)
+        assert os.path.isfile(txtfile)
+
+        # create the filelist, with just the above files
+        myfilelist = [isofile, srtfile, samplefile, txtfile]
+
+        # and now unleash the magic on that filelist, with a more useful jobname:
+        jobname = "My Important Download 2020"
+        self.deobfuscate_wrapper(myfilelist, jobname)
+
+        # Check original files:
+        assert not os.path.isfile(isofile)  # original iso not be there anymore
+        assert not os.path.isfile(srtfile)  # ... and accompanying file neither
+        assert not os.path.isfile(samplefile)  # ... and this one neither
+        assert os.path.isfile(txtfile)  # should still be there: not accompanying
+
+        # Check the renaming
+        assert os.path.isfile(os.path.join(dirname, jobname + ".iso"))  # ... should be renamed to the jobname
+        assert os.path.isfile(os.path.join(dirname, jobname + ".srt"))  # ... should be renamed to the jobname
+        assert os.path.isfile(os.path.join(dirname, jobname + "-sample.iso"))  # ... should be renamed to the jobname
+
+        # Done. Remove (non-empty) directory
+        shutil.rmtree(dirname)
+
     def test_deobfuscate_filelist_full(self):
         # Full test, with a combinantion of files: Test that deobfuscate() works and renames correctly
         # ... but only the files that are in the filelist
@@ -123,14 +173,14 @@ class TestDeobfuscateFinalResult:
         dirname = os.path.join(SAB_CACHE_DIR, "testdir" + str(random.randint(10000, 99999)))
         os.mkdir(dirname)
 
-        # Create a big enough file with a non-useful filename
+        # Create a big enough file with a useless filename
         output_file1 = os.path.join(dirname, "111c1c9e2bdfb5114044bf25152b7eaa.bin")
         create_big_file(output_file1)
         assert os.path.isfile(output_file1)
 
-        # and another one
+        # create a small file. Obfuscated, but should not get renamed
         output_file2 = os.path.join(dirname, "222c1c9e2bdfb5114044bf25152b7eaa.bin")
-        create_big_file(output_file2)
+        create_small_file(output_file2)
         assert os.path.isfile(output_file2)
 
         # create the filelist, with just the above files
@@ -151,13 +201,39 @@ class TestDeobfuscateFinalResult:
 
         # Check original files:
         assert not os.path.isfile(output_file1)  # original filename should not be there anymore
-        assert not os.path.isfile(output_file2)  # original filename should not be there anymore
+        assert os.path.isfile(output_file2)  # original smaller file should still be there
         assert os.path.isfile(output_file3)  # but this one should still be there
         assert os.path.isfile(output_file4)  # and this one too
 
         # Check the renaming
         assert os.path.isfile(os.path.join(dirname, jobname + ".bin"))  # ... it should be renamed to the jobname
-        assert os.path.isfile(os.path.join(dirname, jobname + ".1.bin"))  # should be there (2nd file renamed)
+
+        # Done. Remove (non-empty) directory
+        shutil.rmtree(dirname)
+
+    def test_deobfuscate_one_small_file(self):
+        #  Test of deobfuscating: with just one small file
+
+        # Create directory (with a random directory name)
+        dirname = os.path.join(SAB_CACHE_DIR, "testdir" + str(random.randint(10000, 99999)))
+        os.mkdir(dirname)
+
+        # Create a small file with a useless, obfuscated filename
+        output_file1 = os.path.join(dirname, "blabla.txt")
+        create_small_file(output_file1)
+        assert os.path.isfile(output_file1)
+
+        # create the filelist, with just the above file
+        myfilelist = [output_file1]
+
+        # and now unleash the magic on that filelist, with a more useful jobname:
+        jobname = "My Important Download 2020"
+        self.deobfuscate_wrapper(myfilelist, jobname)
+
+        # Check original files:
+        assert not os.path.isfile(output_file1)  # original filename should not be there anymore
+        # Check the renaming
+        assert os.path.isfile(os.path.join(dirname, jobname + ".txt"))  # ... it should be renamed to the jobname
 
         # Done. Remove (non-empty) directory
         shutil.rmtree(dirname)
@@ -173,7 +249,7 @@ class TestDeobfuscateFinalResult:
         subsubdirname = os.path.join(subdirname, "testdir" + str(random.randint(10000, 99999)))
         os.mkdir(subsubdirname)
 
-        # Create a big enough file with a non-useful, obfuscated filename
+        # Create a big enough file with a useless, obfuscated filename
         output_file1 = os.path.join(subsubdirname, "111c1c9e2bdfb5114044bf25152b7eab.bin")
         create_big_file(output_file1)
         assert os.path.isfile(output_file1)
@@ -202,7 +278,7 @@ class TestDeobfuscateFinalResult:
         os.mkdir(dirname)
         subdirname = os.path.join(dirname, "VIDEO_TS")
         os.mkdir(subdirname)
-        # Create a big enough file with a non-useful, obfuscated filename (which normally should get renamed)
+        # Create a big file with a useless, obfuscated filename (which normally should get renamed)
         output_file1 = os.path.join(subdirname, "111c1c9e2bdfb5114044bf25152b7eab.bin")
         create_big_file(output_file1)
         assert os.path.isfile(output_file1)
@@ -214,69 +290,20 @@ class TestDeobfuscateFinalResult:
         self.deobfuscate_wrapper(myfilelist, jobname)
 
         # ... but because inside "VIDEO_TS" directory, the file should not be touched / renamed:
-        assert os.path.isfile(output_file1)  # should stil be there
+        assert os.path.isfile(output_file1)  # should still be there
 
         # Done. Remove (non-empty) directory
         shutil.rmtree(dirname)
 
-    def test_deobfuscate_big_file_small_accompanying_files(self):
-        # input: myiso.iso, with accompanying files (.srt files in this case)
-        # test that the small accompanying files (with same basename) are renamed accordingly to the big ISO
+    def test_deobfuscate_collection_with_same_size(self):
+        # input: a collection of a few files with about the same size
+        # test that there is no renaming
 
         # Create directory (with a random directory name)
         dirname = os.path.join(SAB_CACHE_DIR, "testdir" + str(random.randint(10000, 99999)))
         os.mkdir(dirname)
 
-        # Create a big enough file with a non-useful filename
-        isofile = os.path.join(dirname, "myiso.iso")
-        create_big_file(isofile)
-        assert os.path.isfile(isofile)
-
-        # and a srt file
-        srtfile = os.path.join(dirname, "myiso.srt")
-        create_small_file(srtfile)
-        assert os.path.isfile(srtfile)
-
-        # and a dut.srt file
-        dutsrtfile = os.path.join(dirname, "myiso.dut.srt")
-        create_small_file(dutsrtfile)
-        assert os.path.isfile(dutsrtfile)
-
-        # and a non-related file
-        txtfile = os.path.join(dirname, "something.txt")
-        create_small_file(txtfile)
-        assert os.path.isfile(txtfile)
-
-        # create the filelist, with just the above files
-        myfilelist = [isofile, srtfile, dutsrtfile, txtfile]
-
-        # and now unleash the magic on that filelist, with a more useful jobname:
-        jobname = "My Important Download 2020"
-        self.deobfuscate_wrapper(myfilelist, jobname)
-
-        # Check original files:
-        assert not os.path.isfile(isofile)  # original iso not be there anymore
-        assert not os.path.isfile(srtfile)  # ... and accompanying file neither
-        assert not os.path.isfile(dutsrtfile)  # ... and this one neither
-        assert os.path.isfile(txtfile)  # should still be there: not accompanying, and too small to rename
-
-        # Check the renaming
-        assert os.path.isfile(os.path.join(dirname, jobname + ".iso"))  # ... should be renamed to the jobname
-        assert os.path.isfile(os.path.join(dirname, jobname + ".srt"))  # ... should be renamed to the jobname
-        assert os.path.isfile(os.path.join(dirname, jobname + ".dut.srt"))  # ... should be renamed to the jobname
-
-        # Done. Remove (non-empty) directory
-        shutil.rmtree(dirname)
-
-    def test_deobfuscate_collection_with_same_extension(self):
-        # input: a collection of 3+ bigger files with the same extension
-        # test that there is no renaming on the collection ... as that's useless on a collection
-
-        # Create directory (with a random directory name)
-        dirname = os.path.join(SAB_CACHE_DIR, "testdir" + str(random.randint(10000, 99999)))
-        os.mkdir(dirname)
-
-        # Create big enough files with a non-useful filenames, all with same extension
+        # Create big enough files with a useless filenames, all with same extension
         file1 = os.path.join(dirname, "file1.bin")
         create_big_file(file1)
         assert os.path.isfile(file1)
@@ -293,13 +320,8 @@ class TestDeobfuscateFinalResult:
         create_big_file(file4)
         assert os.path.isfile(file4)
 
-        # other extension ... so this one should get renamed
-        otherfile = os.path.join(dirname, "other.iso")
-        create_big_file(otherfile)
-        assert os.path.isfile(otherfile)
-
         # create the filelist, with the above files
-        myfilelist = [file1, file2, file3, file4, otherfile]
+        myfilelist = [file1, file2, file3, file4]
 
         # and now unleash the magic on that filelist, with a more useful jobname:
         jobname = "My Important Download 2020"
@@ -311,11 +333,6 @@ class TestDeobfuscateFinalResult:
         assert os.path.isfile(file2)  # still there
         assert os.path.isfile(file3)  # still there
         assert os.path.isfile(file4)  # still there
-        # but the one separate file with obfuscated name should be renamed:
-        assert not os.path.isfile(otherfile)  # should be renamed
-
-        # Check the renaming
-        assert os.path.isfile(os.path.join(dirname, jobname + ".iso"))  # ... should be renamed to the jobname
 
         # Done. Remove (non-empty) directory
         shutil.rmtree(dirname)
@@ -379,6 +396,7 @@ class TestDeobfuscateFinalResult:
         # first par2 based renaming aaaaaaaaaaa to twentymb.bin,
         # then deobfuscate twentymb.bin to the job name (with same extension)
         list_of_files = recover_par2_names(list_of_files)
+        print(list_of_files)
         assert os.path.isfile(os.path.join(work_dir, "twentymb.bin"))  # should exist
 
         self.deobfuscate_wrapper(list_of_files, "My Great Download")
