@@ -1,12 +1,9 @@
 """
 tests.test_nzbstuff - Testing functions in nzbstuff.py
 """
-import io
-
 import sabnzbd.nzbstuff as nzbstuff
 from sabnzbd.config import ConfigCat
 from sabnzbd.constants import NORMAL_PRIORITY
-from sabnzbd.encoding import utob
 from sabnzbd.filesystem import globber
 
 from tests.testhelper import *
@@ -53,6 +50,52 @@ class TestNZO:
         # TODO: More checks!
 
 
+class Server:
+    def __init__(self, host, priority, active):
+        self.host = host
+        self.priority = priority
+        self.active = active
+
+
+class TestArticle:
+    def test_get_article(self):
+        article_id = "test@host" + os.urandom(8).hex() + ".sab"
+        article = nzbstuff.Article(article_id, randint(4321, 54321), None)
+        servers = []
+        servers.append(Server("testserver1", 10, True))
+        servers.append(Server("testserver2", 20, True))
+        servers.append(Server("testserver3", 30, True))
+
+        # Test fetching top priority server
+        server = servers[0]
+        assert article.get_article(server, servers) == article
+        assert article.fetcher_priority == 10
+        assert article.fetcher == server
+        assert article.get_article(server, servers) == None
+        article.fetcher = None
+        article.add_to_try_list(server)
+        assert article.get_article(server, servers) == None
+
+        # Test fetching when there is a higher priority server available
+        server = servers[2]
+        assert article.fetcher_priority == 10
+        assert article.get_article(server, servers) == None
+        assert article.fetcher_priority == 20
+
+        # Server should be used even if article.fetcher_priority is a higher number than server.priority
+        article.fetcher_priority = 30
+        server = servers[1]
+        assert article.get_article(server, servers) == article
+
+        # Inactive servers in servers list should be ignored
+        article.fetcher = None
+        article.fetcher_priority = 0
+        servers[1].active = False
+        server = servers[2]
+        assert article.get_article(server, servers) == article
+        assert article.tries == 3
+
+
 class TestNZBStuffHelpers:
     @pytest.mark.parametrize(
         "argument, name, password",
@@ -140,9 +183,9 @@ class TestNZBStuffHelpers:
                 '[PRiVATE]-[WtFnZb]-[Video_(2001)_AC5.1_-RELEASE_[TAoE].mkv]-[1/23] - "" yEnc 1234567890 (1/23456)',
             ),
             (
-                "[PRiVATE]-[WtFnZb]-[219]-[1/serie.name.s01e01.1080p.web.h264-group.mkv] - "
+                "[PRiVATE]-[WtFnZb]-[219]-[1/series.name.s01e01.1080p.web.h264-group.mkv] - "
                 " yEnc (1/[PRiVATE] \\c2b510b594\\::686ea969999193.155368eba4965e56a8cd263382e012.f2712fdc::/97bd201cf931/) 1 (1/0)",
-                "serie.name.s01e01.1080p.web.h264-group.mkv",
+                "series.name.s01e01.1080p.web.h264-group.mkv",
             ),
         ],
     )
