@@ -28,6 +28,7 @@ import pytest
 from random import choice, randint
 import requests
 from selenium.common.exceptions import WebDriverException
+from selenium.webdriver.common.by import By
 from selenium.webdriver.common.keys import Keys
 from selenium.webdriver.support.ui import WebDriverWait
 from string import ascii_lowercase, digits
@@ -157,9 +158,9 @@ def create_and_read_nzb_fp(nzbdir: str, metadata: Optional[Dict[str, str]] = Non
     return io.BytesIO(nzb_data)
 
 
-def random_name(lenghth: int = 16) -> str:
+def random_name(length: int = 16) -> str:
     """Shorthand to create a simple random string"""
-    return "".join(choice(ascii_lowercase + digits) for _ in range(lenghth))
+    return "".join(choice(ascii_lowercase + digits) for _ in range(length))
 
 
 class FakeHistoryDB(db.HistoryDB):
@@ -242,7 +243,7 @@ class SABnzbdBaseTest:
         self.no_page_crash()
 
     def scroll_to_top(self):
-        self.driver.find_element_by_tag_name("body").send_keys(Keys.CONTROL + Keys.HOME)
+        self.driver.find_element(By.TAG_NAME, "body").send_keys(Keys.CONTROL + Keys.HOME)
         time.sleep(2)
 
     def wait_for_ajax(self):
@@ -257,7 +258,7 @@ class SABnzbdBaseTest:
     @staticmethod
     def selenium_wrapper(func, *args):
         """Wrapper with retries for more stable Selenium"""
-        for i in range(3):
+        for _ in range(3):
             try:
                 return func(*args)
             except WebDriverException as e:
